define(['app', 'uicontrols', 'backbone', 'underscore'], function(App, $c, backbone, _){

App.view.login = App.view.base.extend({
	tagName: 'form',
	className: 'dwc_login-view',
	template:
	'<div class="dwc_login-view_inner">' +
		'<% if(!obj.user){ %>' + 
			'<div class="dwc_login-view_error" data-co="error"></div>' +
			'<input type="text" value="" placeholder="<%=_tools._("Login") %>" data-co="login" class="dwc-form_input dwc_login-view_input" required/>' +
			'<input type="password" value="" placeholder="<%=_tools._("Password")%>" data-co="pass" class="dwc-form_input dwc_login-view_input" />' +
			'<button class="dwc_btn dwc_login-view_submit" name="submit"><%=_tools._("login_btn")%></button>' +
		'<% }else{ %>' +
			'<div class="dwc_login-view_label"><%=obj.user%></div>' +			
			'<button class="dwc_btn dwc_login-view_submit" name="submit"><%=_tools._("logout_btn")%></button>' +
		'<% } %>' +
	'</div>',
	share: {
		_: App.helpers._,
	},
	_lock: false,
	initialize: function(){
		App.view.base.prototype.initialize.call(this);

		this.listenTo(this.model, 'change:user', function(){
			this.$el.empty(); // TODO check for bugs
			this.render();
			this._lock = false;
			this.el.elements.submit.disabled = false;
		});
		this.listenTo(this.model, 'fail', function(code){
			if(this.controls.login){
				this.controls.login.val('');
				this.controls.pass.val('');	
			}
			
			this._lock = false;
			this.el.elements.submit.disabled = false;
			
			var 	notifyText;

			if(code == 'EC_NOT_AUTHORIZED'){ // not_authorized auth_failed
				notifyText = App.helpers._('not_authorized_description');
			}else if(code == 'EC_AUTH_FAILED'){
				notifyText = App.helpers._('auth_failed_decription');
			}else{
				notifyText = App.helpers._(code);
			}
			this.controls.error.text(notifyText);
		});
		this.listenTo(App.i18n, 'change:lang', function(){
			this.render();
		});
		this.on('remove', function(){
			if(this.model){
			   this.model.trigger('clear');
			   this.model.clear({silent: true});
			}
		});
	},
	events: {
		'click': function(e){e.stopPropagation();},
		'submit': function(e){
			e.preventDefault();
			if(this._lock) return;	
			
			this._lock = true;
			this.el.elements.submit.disabled = true;

			if(!this.model.get('user')){
				this.model.autenticate(this.controls.login.val(), this.controls.pass.val());
			}else{
				this.model.logout();
			}
		},
	},
	render: function(){
		App.view.base.prototype.render.call(this);

		// ovveride default validation messages, Validation of pass is turn off by #0115879
		if(this.controls.login){
			var $login = this.controls.login[0];
			$login.onchange = function(e){
				e.target.setCustomValidity('');
				e.target.classList.add('__change');
			};
			$login.oninvalid = function(e){
				e.target.setCustomValidity(App.helpers._('SetYourLogin'));
			};
			setTimeout(function(){
				$login.focus();
			}, 0);
		}
	}
});

App.model.info = Backbone.Model.extend({
	initialize: function(attr, opt){
		this.url = opt.url;
		!attr && opt.defaults && this.set(opt.defaults);	
	},
	loadData: function(){
		return this.fetch({
			error: function(m, r, obj){
				// var code = App.helpers.property('responseJSON.error.code', r);
			}
		});
	}
});

function InitEventSourse(){
	if(typeof(EventSource) == 'undefined') return;

	if(App.source){
		App.source.onmessage = null;	
		App.source.close(); // close previous connection	
	} 
	
	App.source = new EventSource('/api/10.2/eventstream');
	App.source.onmessage = function(e){
		var data = App.helpers.tryBack(function(){
			return JSON.parse(e.data);
		});
		data && App.bus.trigger('refresh', data);
	};
	App.source.onopen = function(e){};
	// Attention don't logout while error event, because it worst at IE
	App.source.onerror = function(e){
		// if(e.target ? e.target.readyState == 2 : App.source.readyState == 2){ // IE doesn't have e.target
		App.user.checkAutorization();
	};
};

var ScanRequestModel = backbone.Model.extend({
	defaults: {
		scan_timeout_ms: 3600,
	    cure: false,
	    heuristic_analysis: true,
	    packer_max_level: 8,
	    archive_max_level: 8,
	    mail_max_level: 8,
	    container_max_level: 8,
	    max_compression_ratio: 500,
	    view_state: 1, // 1 - drop zone (init), 2 - progress, 3 - complete
	},
	getOptions: function(){
		var data = this.toJSON();

		data.scan_timeout_ms *= 1000;
		delete data.view_state;
		return data;
	},
	store: function(){
		var data = this.toJSON();

		delete data.view_state;
		window.localStorage['scanSettings'] = JSON.stringify(data);
	},
	restore: function(){
		var settings;

		try{
			settings = JSON.parse(window.localStorage['scanSettings'] || '');
		}catch(e){}

		if(settings){
			this.set(settings);
		}
	},
	initialize: function(){
		backbone.Model.prototype.initialize.call(this);
		this.restore();
	}
});

var ScanItemModel = backbone.Model.extend({
	defaults: {
		object: '',
		virus: [],
		state: 0, // 1 - founded, 2 - not founded, 3 - error
	},
	// forse set
   	setf: function(key, value){
   		var options = {silent: true};
   		this.set(key, value, options);
   		this.trigger('change:' + key, this, value, options);
   		this.trigger('change', this, options);
   	},
   	getThreat: function(){
   		return this.get('virus').map(function(o){return o.name;}).join(', ');
   	}
});
var ScanItemsCollection = backbone.Collection.extend({
	total: 0,
	isLock: false,
	model: ScanItemModel,
	init: function(fileList, setModel){
		this.isLock = false;

		if(fileList && fileList.length > 0){
			this.total += fileList.length;
			this.readFiles(fileList)
			this.settings = setModel;
		}
	},
	readFiles: function(files){
		var 	MAX_FILE_SIZE = App.i18n.get('maxCheckFileSize') * 1038336 + 1,
				i = files.length,
				fileCollection = [],
				file;

		while(i-- > 0){
			file = files[i];

			if(file.size > MAX_FILE_SIZE){
				this.add({ // report file check result with error
					object: file.name,
					error: 'exceed_file_size_limit',
					state: 2,
				});
			}else{
				fileCollection.push(file);
			}
		}

		this.scanFileQueue(fileCollection, files.length);
	},
	scanFileQueue: function(files, total){
		var file = !this.isLock && files.shift();

		if(file){
			var _model = this.add({
				object: file.name
			});

			this.trigger('check_file', {
				object: file.name,
				index: files.length
			}, this);

			var fr = new FileReader();
			fr.onload = function(e) {      
				var 	sendData = e.target.result;

				// alternative method is subscribe on endpoint ready and endpoint fail events
				this.getEndpoint(function(endPoint, x){
					if(!endPoint){
						_model.set({
							error: x.ec,
							state: 2,
						});
						this.scanFileQueue(files, total);
					}else{
						this.curRequest = App.xhr.objPost({ // send file for check 
							url: '/api/10.2/scan_endpoint/' + endPoint,
							resFormat: 'json',
							oncomplete: function(r, x){
								if(x.ec || x.status < 200 || x.status > 299){
									_model.set({
										error: x.ec,
										state: 2,
									});
								}else if(r && r.scan_report){
									var 	haveErrors = r.scan_report.item && findError(r.scan_report.item),
											haveVirus = !haveErrors && ((Array.isArray(r.scan_report.virus) && r.scan_report.virus.length > 0) || (Array.isArray(r.scan_report.item) && r.scan_report.item.length > 0));

									_model.set({
										error: haveErrors, // find engine errors
										item: r.scan_report.item,
										virus: r.scan_report.virus, 
										state: haveVirus ? 1 : 2
									});
								}else{
									_model.set('state', 2);
								}
								this.scanFileQueue(files, total);
							}.bind(this),
							params: sendData,
						});
					}
				}.bind(this));
	       	}.bind(this);
			fr.onerror = function(e){
				_model.set({ // Attention: folder have empty property 'type'
					error: file.type ? 'file_read_error' : 'folder_read_error',
					state: 2
				});
				this.scanFileQueue(files, total);
			}.bind(this);

		    fr.readAsArrayBuffer(file);
		}else{
			this.total -= files.length;
			this.trigger('complete', this);
		}
	},
	// @param {Function} cb
	getEndpoint: function(cb){
		this.curRequest = App.xhr.post('/api/10.2/scan_request', this.settings.getOptions(), function(d, x){
			if(x.sessionComplete) return;
			cb(d && d.endpoint, x);
		}.bind(this), 'json');
	},
	count: function(){
		var 	res = 0,
				len = this.models.length;

		while(len--){
			if(this.models[len].get('state') == 1){
				res++;
			}
		}

		return res;
	},
	errorCount: function(){
		var 	res = 0,
				len = this.models.length;

		while(len--){
			if(this.models[len].get('error')){
				res++;
			}
		}

		return res;
	},
	abort: function(){
		if(this.curRequest && this.curRequest.abort){
			this.curRequest.abort();
		}
		this.isLock = true;
	}
});

function findError(list){
	var 	i = list.length,
			res = false;

	while(i-- > 0){
		if(list[i].error){ 
			res = true;
		}else{
			if(Array.isArray(list[i].virus) && list[i].virus.length > 0){
				return false;
			}else if(Array.isArray(list[i].item) && list[i].item.length > 0){
				if(findError(list[i].item)){
					res = true;
				}else{
					return false;
				}
			}
		} 
	}
	return res;
}

function ShowThreatsPopup(data){
	this.open(data);
}
ShowThreatsPopup.prototype.open = function(data){
	var		count = this.count(data);
	 		errorCount = this.errorCount(data);

	new $c.BasePopupView({
	 	class: 'dec_wt-popup',
	  	content: 
	  	'<button data-co="close" class="dwc_popup-close"></button>' +
		// TODO rewrite code / change loaclization (move space):
		'<h3 class="dwc_check-res_title ' + (count > 0 ? '__detected' : !errorCount ? '__not-detected' : '__error') + '">' + App.helpers._(count > 0 ? 'check-popup_threat-founded' : errorCount > 0 ? 'check-popup_errors_while_check' : 'check-popup_threat-not-founded') + ' ' + (count > 0 ? count : '') + '</h3>' +
	  	'<div data-co="wrap" class="dwc_chresgrid-wrap"></div>' +
	  	'<div class="dwc_scan-report-footer">' +
	  		'<button class="dwc_btn" data-co="cancel">' + App.helpers._('Complete') + '</button>' +
	  	'</div>' +
	  	'',
	 	onopen: function(view){
		 	view.controls.close.on('click', function(e){
	  			e.stopPropagation();
	  			e.preventDefault();
	  			view.close();
	  		});
	  		view.controls.cancel.on('click', function(e){
	  			view.close();
		 	});

	  		var 	$frag,
	  				$frag2 = document.createDocumentFragment(),
	  				i, state, error, m, items, node, hasError, conf, buf,
	  				level = 0;

	  		for(i = 0, len = data.length; i < len; i++){
	  			state = data[i].state;
	  			error = data[i].error;
	  			items = data[i].item;

	  			conf = {
	  				left: data[i].object
	  			};
	  			

		 		if(Array.isArray(items) && items.length > 0){
		 			node = document.createElement('div');
		 			hasError = findError(items);
		 			node.className = 'dwc_chresgrid-inserted_block';
		 			node.setAttribute('data-level', 0);
		 			// conf.content = this.subGroup(items, 1);
		 			buf = this.subGroup(items, 1);
		 			conf.content = buf.el;
		 			conf.right = App.helpers._(hasError ? 'сheck_with_errors' : 'infected_archive');
		 			conf.lineClass = hasError ? '__error': '__threat';
		 			node.appendChild(this.addFolder(conf));
		 		} else {
		 			if(error){
		 				conf.right = App.helpers._('check_popup_error').replace('%s', App.helpers._(error));
		 				conf.lineClass = '__error';
					}else if(state == 1){ // founded
		 				conf.right = App.helpers._('check_popup_threat').replace('%s', App.helpers._(this.getThreat(data[i])));
		 				conf.lineClass = '__threat';
		 			}else if(state == 2){ // not founded
		 				// conf.right = '\u2014';
		 				conf.right = App.helpers._('Clean');
		 			}
		 			node = this.addFolder(conf);
		 		}
		 		$frag2.appendChild(node);
		 	}

		 	$frag = this.addFolder({
		 		lineClass: '__header',
		 		left: App.helpers._('File_name'),
		 		right: App.helpers._('Result'),
		 		content: $frag2
		 	});
		 	view.controls.wrap.append($frag);
		 }.bind(this),
		 onclose: function(view){},
		 _events: {
		 	'wrap click': function(e){
		 		if(e.target.classList.contains('dwc_chresgrid-inserted_arrow')){
		 			var $arrow = e.target;

		 			if($arrow.classList.contains('__opened')){
						$arrow.classList.remove('__opened')
		 			}else{
	  					$arrow.classList.add('__opened')
	  				}
	  			}
	  		}
	  	}
	  }).open();
}
ShowThreatsPopup.prototype.count = function(data){
	var 	res = 0,
			len = data.length;

	while(len--){
		if(!data[len].state || data[len].state == 1){
			res++;
		}
	}

	return res;
};
ShowThreatsPopup.prototype.errorCount = function(data){
	var 	res = 0,
			len = data.length;

	while(len--){
		if(data[len].error){
			res++;
		}
	}

	return res;
};
ShowThreatsPopup.prototype.getThreat = function(data){
	return data.virus.map(function(o){return o.name;}).join(', ');
}
ShowThreatsPopup.prototype.addFolder = function(conf){
	var 	frag =  document.createDocumentFragment(),
			line = document.createElement('div'),
			arrow = document.createElement('div'),
			cell1 = document.createElement('div'),
			cell2 = document.createElement('div'),
			container = document.createElement('div');

	line.className = 'dwc_chresgrid-line' + (conf.lineClass ? ' ' + conf.lineClass : '');
	cell1.className = 'dwc_chresgrid-cell';
	cell2.className = 'dwc_chresgrid-cell';
	cell1.textContent = conf.left;
	cell2.textContent = conf.right;
	line.appendChild(cell1);
	line.appendChild(cell2);	
	container.className = 'dwc_chresgrid-inserted';

	if(conf.content){
		container.appendChild(conf.content);
		arrow.className = 'dwc_chresgrid-inserted_arrow';
		frag.appendChild(arrow);
	}
	frag.appendChild(line);
	frag.appendChild(container);
	
	return frag;
};
ShowThreatsPopup.prototype.subGroup = function(list, level){
	var 	$frag = document.createDocumentFragment(),
			i = list.length,
			conf, node, status, buf;

	while(i-- > 0){
		conf = {
			left: list[i].object,
		};

		if(Array.isArray(list[i].item) && list[i].item.length > 0){
			buf = this.subGroup(list[i].item, level + 1);
			// conf.content = this.subGroup(list[i].item, level + 1);
			conf.content = buf.el;
			status = buf.status;

			if(buf.status == 'error'){
				conf.right = App.helpers._('сheck_with_errors');
				conf.lineClass = '__error';
			}else if(buf.status == 'threat'){
				conf.right = App.helpers._('infected_archive');
				conf.lineClass = '__threat';
			}
		}else{
			if(Array.isArray(list[i].virus) && list[i].virus.length > 0){
				conf.right = App.helpers._('check_popup_threat').replace('%s', App.helpers._(list[i].virus.map(function(v){return v.name;}).join(', ')));
				conf.lineClass = '__threat';
				status = 'threat';
			}else if(list[i].error){
				conf.right = App.helpers._('check_popup_error').replace('%s', App.helpers._(list[i].error));
				conf.lineClass = '__error';
				status = 'error';
			}else{ // this case wasn't founded
				conf.right = '';
			}	
		}
		
		node = document.createElement('div');
		node.className = 'dwc_chresgrid-inserted_block';
		node.setAttribute('data-level', level);
		node.appendChild(this.addFolder(conf));
		$frag.appendChild(node);
	}

	// return $frag;
	return {
		el: $frag,
		status: status
	};
};

App.ShowThreatsPopup = ShowThreatsPopup;	

//////////////////////////////////////////////////
// Installed
//////////////////////////////////////////////////
var InstalledModel = Backbone.Model.extend({
	idAttribute: 'app',
});
var InstalledCollection = Backbone.Collection.extend({
	model: InstalledModel,
	url: '/api/10.2/installed',
	parse: function(r){
		return r.installed;
	},
	getIniMap: function(){
		var 	i = this.models.length;

		if(i > 0){
			var 	map = {};

			while(i-- > 0){
				map[this.models[i].get('ini_section')] = this.models[i].get('app');
			}
			return map;	
		}
	},
});
App.collection.installed = new InstalledCollection();

// In dev
function saveParse(str){
	var parsed = parseInt(str.replace(/\D/g, ''));
	return parsed > Number.MAX_SAFE_INTEGER ? Number.MAX_SAFE_INTEGER : parsed;
}

App.view.index = App.view.base.extend({
	template:
	'<div class="dwc_main" data-co="index-screen" style="display:none;">' +
		'<div class="dwc_main-panel_wrap">' +
			'<div class="dwc_main-panel">' +
				'<div class="dwc_navigation">' +
					'<a class="dwc_nav-item dwc_nav-logo" href="#main" tabindex="0"></a>' +
					'<label class="dwc_nav-item"><input type="radio" class="dwc_nav-radio" value="main" checked name="dwc-nav-radio" data-co="nav-main" tabindex="-2"/><a href="#main" class="dwc_navbtn dwc_navbtn_dashboard"><span class="dwc_navbtn-label" data-i18="mainLink">' + App.helpers._('mainLink') + '</span></a></label>' +
					'<label class="dwc_nav-item"><input type="radio" class="dwc_nav-radio" value="threat" name="dwc-nav-radio" data-co="nav-threats" /><a href="#threats" class="dwc_navbtn dwc_navbtn_threats" data-co="threat-link"><span class="dwc_navbtn-label" data-i18="threatening">' + App.helpers._('threatening') + '</span></a></label>' +
					'<label class="dwc_nav-item"><input type="radio" class="dwc_nav-radio" value="settings" name="dwc-nav-radio" data-co="nav-settings"/><a href="#settings" class="dwc_navbtn dwc_navbtn_settings"><span class="dwc_navbtn-label" data-i18="Settings">' + App.helpers._('Settings') + '</span></a></label>' +
					'<label class="dwc_nav-item"><input type="radio" class="dwc_nav-radio" value="info" name="dwc-nav-radio" data-co="nav-info"/><a href="#info" class="dwc_navbtn dwc_navbtn_info"><span class="dwc_navbtn-label" data-i18="Information">' + App.helpers._('Information') + '</span></a></label>' +
					'<label class="dwc_nav-item"><a href="#" target="_blank" class="dwc_navbtn dwc_navbtn_help" data-co="help-link"><span class="dwc_navbtn-label" data-i18="Help">' + App.helpers._('Help') + '</span></a></label>' +
					'<label class="dwc_nav-item __space"></label>' +
					'<label class="dwc_nav-item" data-co="maild-pack-pass-btn" style="display:none;"><a href="javascript:void(0)" class="dwc_navbtn dwc_navbtn_maildpass">' +
						'<span class="dwc_navbtn-label" data-i18="navlabel_maild-pack-pass">' + App.helpers._('navlabel_maild-pack-pass') + '</span>' +
					'</a></label>' +
					'<label class="dwc_nav-item"><a href="javascript:void(0)" class="dwc_navbtn dwc_navbtn_check co-check-btn"><span class="dwc_navbtn-label" data-i18="checkFile">' + App.helpers._('checkFile') + '</span></a></label>' +
					'<label class="dwc_nav-item"><a href="javascript:void(0)" class="dwc_navbtn dwc_navbtn_logout"><span class="dwc_navbtn-label" data-i18="logout_btn">' + App.helpers._('logout_btn') + '</span></a></label>' +
				'</div>' +
			'</div>' +
			'<div class="dwc_main-panel_back"></div>' +
		'</div>' +
		'<div class="dwc_main-body clearfix" data-co="content"></div>' +
		'<div class="dwc_main-dropzone" data-co="dropzone" style="display:none;"></div>' +
	'</div>' +
	'<div class="dwc_main" data-co="init-screen"></div>' +
	'<div class="dwc_start-screen clearfix" data-co="login-screen" style="display:none;">' +
			'<div data-co="login-form" class="dwc_content-login_form"></div>' +
		'<div class="dwc_start-screen_footer">'+
			'<a href="http://www.drweb.com" target="_blank" data-i18="DrWeb_site" data-i18-attr="href:drweb_site_link"></a>' +
			'<a href="http://company.drweb.com/policy" target="_blank" data-i18="Privacy_policy"></a>' +
			'<a href="http://support.drweb.com" target="_blank" data-i18="Technical_support"></a>' +
			'<a href="#" target="_blank" data-i18="Help" data-co="login-help-link"></a>' +
		'</div>' +
	'</div>',
	events: {
		'click .dwc_navbtn_logout': function(e){
			App.user.logout();
		},
		'click .co-check-btn': function(e){
			this.checkFile();
		},
		'dragstart': function(e){ 
			// prevent dropping page content (see #113107)
			e.originalEvent.dataTransfer.effectAllowed = 'none';
		},
		// Fire only on page content, because menu have hover effect when screen less 1024px width
		'dragenter [data-co=content]': function(e){
			e.stopPropagation();
			e.preventDefault();

			if(
				typeof(e.originalEvent.dataTransfer.effectAllowed) != 'string' || // to avoid exceptions at IE
				e.originalEvent.dataTransfer.effectAllowed != 'none' // Best at all browsers beside IE. to prevent drop content from page
			){ 
				this.controls.dropzone.show();
			}
		},
		// prevent open file at page
		'dragover [data-co=dropzone]': function(e){
			e.stopPropagation();
			e.preventDefault();
		},
		'dragleave [data-co=dropzone]': function(e){
			this.controls.dropzone.hide();
		},
		'drop [data-co=dropzone]': function(e){
			e.stopPropagation();
			e.preventDefault();
			var 	files = e.originalEvent.dataTransfer.files;

			this.checkFile(files);
			this.controls.dropzone.hide();
		},
		'click [data-co=maild-pack-pass-btn]': function(e){
			this.controls.maildPackPassBtn.show();
				
			if(!this.maildPassPopup){
				require(['maild_pass.popup'], function(popup){
					this.maildPassPopup = popup;
					// Open imported popup
					this.maildPassPopup.open()
				}, function(e){
					console.warn('Fail passwd popup');
					console.log(e);
				});
			}else{
				// Open existed popup
				this.maildPassPopup.open();
			}
		},
	},
	checkFile: function(_fileList){
		var popup = this.checkPopupInstance;

		if(popup && _fileList && _fileList.length > 0){
			popup.model.set('view_state', 2);
			popup.collection.init(_fileList, popup.model);
			return;
		}

		var templateBody =
            '<form class="dwc_check-wrap" data-co="drop-layer">' + // canvas1
                '<label class="dwc_check-dropzone" data-co="drop-zone">' +
                    '<div class="m3_middle dwc_check-dropzone_label" data-co="drop-zone-label">' +
                        App.helpers._('check-popup_help-message').replace('%1', App.i18n.get('maxCheckFileSize')) +
                    '</div>' +
                    '<span class="m3_middle_helper"></span>' +
                    '<input type="file" data-co="file" class="dwc_check-dropzone_input" multiple/>' +
                '</label>' +
            '</form>';

		var netCheckInstalled = App.collection.installed._byId['APP_NET_CHECK'] && true;
        if (!netCheckInstalled) {
            templateBody =
                '<div class="dwc_check-wrap">' +
                    '<div class="help-link_msg">' +
                        App.helpers._('NC_is_not_installed_msg') +
                    '</div>' +
                    '<div class="help-link_label"></div>' +
                '</div>';
        }

		this.checkPopupInstance = new $c.BasePopupView({
			class: 'dwc_pwrap1',
			model: new ScanRequestModel(),
			collection: new ScanItemsCollection(),
			template:
'<div data-co="content">' +
'<div data-co="body" class="dwc_pwrap6">' +
	// hidden input
	'<input type="checkbox" id="dec-conf-checkbox" data-co="conf" class="dec_check-popup_checkbox"/>' +
	// config:
	'<div class="dwc_conf-wrap">' +
		'<div class="dwc_conf-content m3_middle">' +
			'<h3 class="dwc_check-title" data-i18="check-popup_settings-title">' + App.helpers._('check-popup_settings-title') + '</h3>' +
			'<table class="dec_check-popup_grid" data-co="control-grid">' +
				'<tbody>' +
				'<tr><td data-i18="scan_timeout_ms_liter">' + App.helpers._('scan_timeout_ms_liter') +'</td><td>' +
					'<span class="dwc_visual-input" data-component="visualinput" data-empty="0" contenteditable data-postfix="' + App.helpers._('sec') + '" data-co="scan-time-input"></span>' +
				'</td></tr>' +
				'<tr><td data-i18="heuristic_analysis_liter">' + App.helpers._('heuristic_analysis_liter') + '</td><td>' +
					'<label class="dwc_chbox"><input type="checkbox" data-co="heuristic-checkbox"><span class="dwc_chbox-icn" ></span></label>' +
				'</td></tr>' +
				'<tr><td data-i18="max_compression_ratio_liter">' + App.helpers._('max_compression_ratio_liter') +'</td><td>' +
					'<span class="dwc_visual-input" data-component="visualinput" contenteditable data-co="compression-input" data-empty="2"></span>' +
				'</td></tr>' +
				'<tr class="dec_check-popup_row2"><td data-i18="maximum_level_of_nesting_sites">' + App.helpers._('maximum_level_of_nesting_sites') + '</td><td>&nbsp;' +
				'</td></tr>' +
				'<tr class="dec_check-popup_row2"><td data-i18="packer_max_level_liter">' + App.helpers._('packer_max_level_liter') + '</td><td>' +
					'<span class="dwc_visual-input" data-component="visualinput" contenteditable data-co="packer-input" data-empty="0"></span>' +
				'</td></tr>' +
				'<tr class="dec_check-popup_row2"><td data-i18="archive_max_level_liter">' + App.helpers._('archive_max_level_liter') + '</td><td>' +
					'<span class="dwc_visual-input" data-component="visualinput" contenteditable data-co="archive-input" data-empty="0"></span>' +
				'</td></tr>' +
				'<tr class="dec_check-popup_row2"><td data-i18="mail_max_level_liter">' + App.helpers._('mail_max_level_liter') + '</td><td>' +
					'<span class="dwc_visual-input" data-component="visualinput" contenteditable data-co="mail-input" data-empty="0"></span>' +
				'</td></tr>' +
				'<tr class="dec_check-popup_row2"><td data-i18="container_max_level_liter">' + App.helpers._('container_max_level_liter') +'</td><td>' +
					'<span class="dwc_visual-input" data-component="visualinput" contenteditable data-co="container-input" data-empty="0"></span>' +
				'</td></tr>' +
				'</tbody>' +
			'</table>' +
			'<div class="dwc-form_btn-wrap">' +
				'<label for="dec-conf-checkbox" class="dwc_btn" data-co="cancel-conf" data-i18="Cancel" tabindex="0">' + App.helpers._('Cancel') + '</label>' +
				'<label for="dec-conf-checkbox" class="dwc_btn __green" data-co="apply-conf" data-i18="Apply" tabindex="0">' + App.helpers._('Apply') + '</label>' +
			'</div>' +
		'</div>' +
		'<span class="m3_middle_helper"></span>' +
	'</div>' +
	// scan states:
	'<div class="dwc_scan-wrap">' +
        '<label data-co="conf-nav-btn" class="dwc_popup-conf" for="dec-conf-checkbox" tabindex="0"></label>' +
        '<button data-co="close" class="dwc_popup-close"></button>' +
        '<h3 class="dwc_check-title dwc_check-title_compact" data-i18="check-popup_title">' +
            App.helpers._('check-popup_title') +
        '</h3>' +
        templateBody +
		'<div class="clearfix dec_check-results" data-co="scan-layer" style="display: none;">' +
			'<h3 class="dwc_check-res_title"><span class="" data-co="scan-title"></span> <span class="" data-co="scan-count"></span></h3>' +
			'<div class="dec_check-popup_repline" data-co="total-files-line" style="display:none;">' + App.helpers._('check-popup_loaded') + ' <span class="dec_check-popup_repline-inner" data-co="total-file-count"></span></div>' +
			'<div class="dec_check-popup_repline" data-co="current-file-line" style="display:none;">' + App.helpers._('check-popup_file-column') + ' <span class="dec_check-popup_repline-inner" data-co="current-file-name"></span></div>' +
			'<div class="dec_check-popup_repline" data-co="threat-line" style="display:none;">' + App.helpers._('check-popup_threat-column') + ' <span class="dec_check-popup_repline-inner" data-co="threat-name"></span></div>' +
			'<div class="dec_check-popup_repline" data-co="error-line" style="display:none;">' + App.helpers._('check-popup_error-column') + ' <span class="dec_check-popup_repline-inner" data-co="error-title"></span></div>' +
			'<div class="dec_check-popup_more-btn" data-co="info-btn" style="display:none;">' + App.helpers._('check-popup_report-link') + ' </div>' +
			'<button data-co="prevent-check-btn" class="dwc_btn dec_check-popup_stop-btn">' + App.helpers._('stop_check') + '</button>' +
		'</div>' +
	'</div>' +
'</div>' +
'</div>',
			onopen: function(view){
			    if (!netCheckInstalled) {
			        view.$el.find('.help-link_label').append(
                        App.helpers.patterns.parseStrWithNodes(App.helpers._('NC_is_not_installed_link'), {
                            '%linkAddr%': '/help/' + App.i18n.helpSegment() + '/drweb-configd/index.html?dw_9_errors.htm'
                        })
                    );
                }
				view.sendFiles = function(files){
					if(files && files.length > 0){
						// TODO refactor this - set referense on model
						this.collection.checkFilesConf = this.model.toJSON(
							);
						delete this.collection.checkFilesConf.view_state; // can kill request
						this.collection.init(files, this.model);
					}
				};

				view.controls.scanTimeInput.text(view.model.get('scan_timeout_ms'));
				view.controls.heuristicCheckbox.attr('checked', view.model.get('heuristic_analysis'));
				view.controls.packerInput.text(view.model.get('packer_max_level'));
				view.controls.archiveInput.text(view.model.get('archive_max_level'));
				view.controls.mailInput.text(view.model.get('mail_max_level'));
				view.controls.containerInput.text(view.model.get('container_max_level'));
				view.controls.compressionInput.text(view.model.get('max_compression_ratio'));

				// Attention IE don't fire input event! (Fix on Edge)
				view.controls.controlGrid.on('input', function(e){
					// TODO fix cursor position
					if(e.target.dataset && e.target.dataset.component == 'visualinput'){
						if(window.getSelection && document.createRange){
							var 	text = (saveParse(e.target.textContent) || '') + ''; // <- TEST
							var	 	sel = window.getSelection(),
									range = sel.getRangeAt(0),
									preCaretRange = range.cloneRange(),
									pos;

							preCaretRange.selectNodeContents(e.target);
							preCaretRange.setEnd(range.endContainer, range.endOffset);
							pos = preCaretRange.toString().length - sel.toString().length;

							if(pos > text.length){
								pos = text.length;
							}else if(text != e.target.textContent && pos > 0 ){
								pos--;
							}
							e.target.textContent = text;

							if(e.target.childNodes[0]){
								range = document.createRange();
								range.setStart(e.target.childNodes[0], pos);
								range.setEnd(e.target.childNodes[0], pos);
								range.collapse(false);
								sel.removeAllRanges();
								sel.addRange(range);	
							}
						}else{ // need test at Edge
							e.target.textContent = (saveParse(e.target.textContent) || '') + '';
						}
					}
				});
				// Hello IE:
				var _lock = false;
				view.controls.controlGrid.on('textinput', function(e){
					if(!_lock && e.target.dataset && e.target.dataset.component == 'visualinput'){
						_lock = true;
						var		range = window.getSelection().getRangeAt(0);
								pos = range.startOffset,
								sel = document.createRange(),
								text = (saveParse(e.target.textContent) || '') + '';

						if(pos > text.length){
							pos = text.length;
						}else if(text != e.target.textContent && pos > 0 ){
							pos--;
						}
						e.target.textContent = text;

						if(e.target.childNodes[0]){
							range = document.createRange();
							range.setStart(e.target.childNodes[0], pos);
							range.setEnd(e.target.childNodes[0], pos);
							var selection = window.getSelection();
							selection.addRange(range);
						}

						_lock = false;
					}
				});
				view.controls.content.on('click', function(e){
					e.stopPropagation();
				});
				view.controls.close.on('click', function(e){
					e.stopPropagation();
					e.preventDefault();
					view.close();
				});
				// DROP events only for installed
                if (netCheckInstalled) {
                    view.controls.file.on('change', function(e){
                        view.collection.init(e.target.files, view.model);
                        view.controls.dropLayer[0].reset();
                    });
                    view.controls.file.on('focus', function(e){
                        view.controls.dropZone.addClass('__drop');
                    });
                    view.controls.file.on('blur', function(e){
                        view.controls.dropZone.removeClass('__drop');
                    });
                    view.controls.dropZone.on('dragenter', function(e){
                        e.stopPropagation();
                        e.preventDefault();
                        view.controls.dropZone.addClass('__drop');
                    });
                    view.controls.dropZone.on('dragover', function(e){
                        e.stopPropagation();
                        e.preventDefault();
                    });
                    view.controls.dropZone.on('dragleave', function(e){
                        view.controls.dropZone.removeClass('__drop');
                    });
                    view.controls.dropZone.on('drop', function(e){
                        e.stopPropagation();
                        e.preventDefault();
                        view.collection.init(e.originalEvent.dataTransfer.files, view.model);
                    });
                    view.collection.on('add', function(m, c){
                        view.model.set('view_state', 2);
                    });
                    view.collection.on('check_file', function(d, c){
                        // view.controls.currentFileName.text(d.name);
                        view.controls.currentFileName.text(d.object);
                        view.controls.currentFileLine.show();
                        view.controls.totalFileCount.text(App.helpers._('%1 from %2').replace('%1', c.total - d.index).replace('%2', c.total));
                        view.controls.totalFilesLine.show();
                        view.controls.errorLine.hide();
                    });
                    view.collection.on('complete', function(c){
                        var count = c.count(),
                            errorCount = c.errorCount(),
                            model = c.models[0];

                        view.model.set('view_state', 3);
                        view.controls.scanTitle.text(App.helpers._(count > 0 ? 'check-popup_threat-founded' : errorCount > 0 ? 'check-popup_errors_while_check': 'check-popup_threat-not-founded'));
                        view.controls.scanCount.text(count > 0 ? count : '');
                        view.controls.scanTitle.parent().addClass(count > 0 ? '__detected': !errorCount ? '__not-detected' : '__error');

                        // Attention count only checked files! Files with size over limit does not counted
                        if (
                            c.models.length > 1 || // show at popup if few files or compressed file
                            model && ( model.get('item') ? model.get('item').length > 0 : model.has('archive'))
                        ) {
                            view.controls.infoBtn.show();
                            // view.controls.totalFilesLine.show();
                            view.controls.totalFileCount.text(c.models.length);

                            if(count > 0){
                                view.controls.infoBtn.addClass('__threat');
                            }
                            view.controls.currentFileLine.hide();
                        } else {
                            var 	threat = model && model.getThreat();

                            if(threat){
                                view.controls.threatLine.show();
                                view.controls.threatName.text(threat);
                            }else if(errorCount > 0){
                                view.controls.errorLine.show();
                                view.controls.errorTitle.text(App.helpers._(model.get('error') === true ? 'сheck_with_errors' : model.get('error'))); // 'exceed_file_size_limit, EC_<error code>, file_read_error, folder_read_error'
                            }
                            view.controls.totalFilesLine.hide();
                        }
                    });
                    view.controls.infoBtn.on('click', function(){
                        new App.ShowThreatsPopup(view.collection.toJSON());
                    });
                    view.controls.preventCheckBtn.on('click', function(){// abort check process
                        view.model.set('view_state', 1);
                        view.collection.abort();
                    });
                    view.listenTo(view.model, 'change:view_state', function(m, state){
                        view.controls.scanCount.text('');

                        if (state == 1) { // init state
                            view.controls.dropLayer.show();
                            view.controls.scanLayer.hide();
                        } else if(state == 2) { // scanning
                            view.controls.scanTitle.text(App.helpers._('check-popup_progress-title'));
                            view.controls.scanTitle.parent().removeClass('__detected').removeClass('__not-detected').removeClass('__error');
                            view.controls.dropLayer.hide();
                            view.controls.scanLayer.show();
                            view.controls.totalFilesLine.hide();
                            view.controls.threatLine.hide();
                            view.controls.infoBtn.hide();
                            view.controls.preventCheckBtn.text(App.helpers._('stop_check'));
                        } else if(state == 3) { // complete
                            view.controls.dropLayer.hide();
                            view.controls.scanLayer.show();
                            view.controls.preventCheckBtn.text(App.helpers._('Complete'));
                        }
                    });

                    if (_fileList && _fileList.length > 0) {
                        view.model.set('view_state', 2);
                        view.collection.init(_fileList, view.model);
                    } else {
                        view.model.set('view_state', 1);
                    }
                }
			},
			onclose: function(view){
				var i = view.collection.models.length;

				view.collection.off();
				while(i--){
					view.collection.models[i].destroy();
				}
				// Attention `this` is indexPage view not popup view!
				this.checkPopupInstance = null;

			}.bind(this),
			_events: {
				'applyConf click': function(){
					// close settings with saving
					this.model.set({
						scan_timeout_ms: parseInt(this.controls.scanTimeInput.text() || '0'),
						heuristic_analysis: this.controls.heuristicCheckbox.prop('checked'),
						packer_max_level: parseInt(this.controls.packerInput.text() || '0'),
						archive_max_level: parseInt(this.controls.archiveInput.text() || '2'),
						mail_max_level: parseInt(this.controls.mailInput.text() || '0'),
						container_max_level: parseInt(this.controls.containerInput.text() || '0'),
						max_compression_ratio: parseInt(this.controls.compressionInput.text() || '0'),
					});
					this.model.store();
				},
				'conf change': function(e){
					var 	co = this.controls,
							mo = this.model;

					if(e.target.checked){ // switch to settings
						co.scanTimeInput.text(mo.get('scan_timeout_ms'));
						co.heuristicCheckbox.attr('checked', mo.get('heuristic_analysis'));
						co.packerInput.text(mo.get('packer_max_level'));
						co.archiveInput.text(mo.get('archive_max_level'));
						co.mailInput.text(mo.get('mail_max_level'));
						co.containerInput.text(mo.get('container_max_level'));
						co.compressionInput.text(mo.get('max_compression_ratio'));
					}
				}
			}
		}).open();

////	 Moving hook #0117445
/*
	this.checkPopupInstance.el.classList.add($c._MOVEABLE_CLASS);
	this.checkPopupInstance.el.style.bottom = 'auto';
	this.checkPopupInstance.el.style.minHeight = '180px';
	this.checkPopupInstance.el.style.right = 'auto';
	this.checkPopupInstance.el.style.left = document.body.clientWidth - 480 + 'px';
	this.checkPopupInstance.el.style.top = '0';
	var $header = this.checkPopupInstance.controls.content[0];

	$(this.checkPopupInstance.el).find('.dwc_conf-wrap, .dwc_scan-wrap').on('mousedown', function(e){
		e.stopPropagation();
	});
	this.checkPopupInstance.controls.close.on('mousedown', function(e){
		e.stopPropagation();
	});
	this.checkPopupInstance.controls.confNavBtn.on('mousedown', function(e){
		e.stopPropagation();
	});
	this.checkPopupInstance.dragEngine = new $c.helpers.MouseMovingController(this.checkPopupInstance.el, $header);
	this.checkPopupInstance.open();
*/
		
	},
	initialize: function(){
		App.view.base.prototype.initialize.call(this);

		this.listenTo(App.router, 'all', function(route, e){
			if(route == 'route') return; // skip default `empty` event
			
			if(!App.user.get('user')){ // if not autorized navigate on login page
				route = 'login';
				App.router.navigate(route);
			}
			
			if(route != 'route:threats' && this.children.threatsPage){
				this.children.threatsPage.threatCollection.pauseFetch();
			}
			
			switch(route){
				case 'route:main':
					this.switchScreen('indexScreen');

					if (!this.children.mainPage) {
						require(['mainPage'], function(mainView){
							this.addSubView('content', new mainView(), 'mainPage');
							App.helpers.localize(this.el);
							this.children.mainPage.$el.siblings().hide();
							App.bus.once('logout', function(){// onlogout clean instance
								this.children.mainPage && this.children.mainPage.remove();
								this.children.mainPage = null;
							}.bind(this));
						}.bind(this), function(err){ 
							App.user.checkAutorization();
						});
					} else {
						this.children.mainPage.$el.show().siblings().hide();
					}
					this.controls.navMain.prop('checked', true);
					break;
				case 'route:threats':
					this.switchScreen('indexScreen');

					// If your want rebuild all the time:
					// this.children.threatsPage && this.children.threatsPage.remove();
					// require(['threatsPage'], function(threatsView){
					// 	if(App.DEBUG){
					// 		console.log('CREATE threatPage; is exist %s', !!this.children.threatsPage);
					// 	}
					// 	this.addSubView('content', new threatsView(), 'threatsPage');
					// 	this.children.threatsPage.$el.siblings().hide();
					// }.bind(this));
					
					if (!this.children.threatsPage) {
						require(['threatsPage'], function(threatsView){
							if (!this.children.threatsPage) { // Attention: There exist one case when first condition not working (maybe slow load process)
								this.addSubView('content', new threatsView(), 'threatsPage');
								App.helpers.localize(this.el);
								this.children.threatsPage.$el.siblings().hide();
								App.bus.once('logout', function(){// onlogout clean instance
									this.children.threatsPage && this.children.threatsPage.remove();
									this.children.threatsPage = null;
								}.bind(this));
							}
						}.bind(this), function(err){
							App.user.checkAutorization();
						});
					} else {
						this.children.threatsPage.$el.show().siblings().hide();
						this.children.threatsPage.threatCollection.resumeFetch();
					}

					this.controls.navThreats.prop('checked', true);
					break;
				case 'route:settings':
					// e - is subpage
					this.switchScreen('indexScreen');
					if (!this.children.settingsPage) {
						require(['settingsPage'], function(settingsView){
							this.addSubView('content', new settingsView({
								page: e
							}), 'settingsPage');
							App.helpers.localize(this.el);
							this.children.settingsPage.$el.siblings().hide();
							App.bus.once('logout', function(){ // onlogout clean instance
								this.children.settingsPage && this.children.settingsPage.remove();
								this.children.settingsPage = null;
							}.bind(this));
						}.bind(this), function(err){

							App.user.checkAutorization();
						});
					} else {
						this.children.settingsPage.$el.show().siblings().hide();
						if(e && this.children.settingsPage && this.children.settingsPage.lexCollection){
							this.children.settingsPage.lexCollection.activate(e);
						}
					}
					this.controls.navSettings.prop('checked', true);
					break;
				case 'route:info':
					this.switchScreen('indexScreen');
					if (!this.children.infoPage) {
						require(['infoPage'], function(infoView){
							this.addSubView('content', new infoView(), 'infoPage');
							// this.localize();
							App.helpers.localize(this.el);
							this.children.infoPage.$el.siblings().hide();
							App.bus.once('logout', function(){ // onlogout clean instance
								this.children.infoPage && this.children.infoPage.remove();
								this.children.infoPage = null;
							}.bind(this));
						}.bind(this), function(err){
							App.user.checkAutorization();
						});
					} else {
						this.children.infoPage.$el.show().siblings().hide();
					}
					this.controls.navInfo.prop('checked', true);
					break;
				default:
					this.switchScreen('loginScreen');
					App.helpers.localizeAttr(this.controls.loginScreen[0]);
					break;
			}
		});

		this.listenTo(App.user, 'change:user', function(model, user){			
			var threats_count = App.i18n.attributes.threats_count;
			var url = '/api/10.2/threats';
			var count = 0;			
			var $link = this.controls.threatLink;

			if(user!=='' || threats_count===undefined){
				App.xhr.get(url, {}, function(d, r){
					if(Array.isArray(d)){
						count= d.length;
						activeThreatening(count)
					} else if(r.sessionComplete){
						return;	
				}})
			}

			function activeThreatening(count){
				if(count > 0){
					$link.attr('data-total', true);
				}else{
					$link.removeAttr('data-total');
				}
			}
		})
	
		/*this.listenTo(App.user, 'change:user', function(m, user){
			var updRoute = Backbone.history.fragment || 'login'; // last statement exist if we reload page and get empty hash

			if(!user){
				updRoute = 'login';
				var 	len = $c.BasePopupView.prototype.stack.length,
						popup;
				
				while(len--){
					popup = $c.BasePopupView.prototype.stack[len];
					popup && popup.close && popup.close();
				}
			}else if(updRoute == 'login'){ // go to main if just logined
				updRoute = 'main';
			}

			if(user){ // turn on if log in
				!App.source && InitEventSourse();
			}else{ // log out
				var 	popups = $c.BasePopupView.prototype.stack,
						popupIndex = popups.length;

				while(popupIndex--){ // remove all opened popups
					popups[popupIndex].remove();
				}
				App.bus.trigger('logout'); // TRIG logout
				
				if(App.source){
					App.source.close();
					App.source = null;
				}
			}

			Backbone.history.fragment = '';
			App.router.navigate(updRoute, {trigger: true});
		});*/

		this.listenTo(App.user, 'change:user', function(m, user){
			var 	updRoute = Backbone.history.fragment || 'login'; // last statement exist if we reload page and get empty hash

			if(user){ // turn on if log in
				if(updRoute == 'login'){ // go to main if just logined
					updRoute = 'main';
				}
				!App.source && InitEventSourse();
			}else{ // log out
				updRoute = 'login';

				var 	popups = $c.BasePopupView.prototype.stack,
						popupIndex = popups.length;

				while(popupIndex-- > 0){ // remove all opened popups
					popups[popupIndex].close && popups[popupIndex].close();
					popups[popupIndex].remove();

				}
				App.bus.trigger('logout'); // TRIG logout
				
				if(App.source){
					App.source.close();
					App.source = null;
				}
			}

			Backbone.history.fragment = '';
			App.router.navigate(updRoute, {trigger: true});
		});
		App.user.checkAutorization();

		this.addSubView('loginForm', new App.view.login({
			model: App.user
		}));
		this.listenTo(App.i18n, 'change:lang', function(model, lang){
			App.helpers.localize(this.el);
			App.helpers.localizeAttr(this.el);
			this.controls.helpLink.prop('href', '/help/' + App.i18n.helpSegment() + '/index.html');
			this.controls.loginHelpLink.prop('href', '/help/' + App.i18n.helpSegment() + '/index.html');

			if(this.checkPopupInstance){ // push localization at check popup
				if(this.checkPopupInstance.model.get('view_state') == 3){
					this.checkPopupInstance.collection.trigger('complete', this.checkPopupInstance.collection);
				}

				this.checkPopupInstance.controls.dropZoneLabel.text(App.helpers._('check-popup_help-message').replace('%1', App.i18n.get('maxCheckFileSize')));
				App.helpers.localize(this.checkPopupInstance.controls.body[0]);
			}
			document.title = App.helpers._('page_title');
		});
		this.listenTo(App.i18n, 'change:threats_count', function(model, count){
			var 	$link = this.controls.threatLink;

			if(count > 0){
				$link.attr('data-total', true);
			}else{
				$link.removeAttr('data-total');
			}
		}.bind(this));
		this.controls.helpLink.prop('href', '/help/' + App.i18n.helpSegment() + '/index.html');
		this.controls.loginHelpLink.prop('href', '/help/' + App.i18n.helpSegment() + '/index.html');
		App.helpers.localize(this.el);
		document.title = App.helpers._('page_title');

		var _sessionCheker = _.debounce(function(){
			App.user.checkAutorization();
		}, 60000);
		document.addEventListener('click', function(){
			_sessionCheker();
		});

		this.listenTo(App.bus, 'refresh', function(e){
			if(e.event == 'installed'){
				App.collection.installed.fetch({reset: true});
			}else if(e.event == 'threats'){
				App.i18n.set('threats_count', e.event_data && e.event_data.total);
			}
		});

		this.listenTo(App.bus, 'disconnect', function(){
			new $c.BasePopupView({
				class: 'dwc_disconnect-popup',
				template: 
					'<div data-co="content" class="dwc_disconnect-label">' + App.helpers._('server_disconnect') + '</div>',
				onopen: function(view){
						view.el.onclick = function(){
						view.close();
					}
					setTimeout(function(){
						view.close();
					}, 6000);
				},
				onclose: function(view){}
			}).open();
		});
		this.listenTo(App.collection.installed, 'reset', function(c){
			if(c.get('APP_MAILD')){
				this.controls.maildPackPassBtn.show();
			}else{
				this.controls.maildPackPassBtn.hide();
			}
		});
		App.collection.installed.fetch({reset: true});
	},
	switchScreen: function(screenId){
		this.controls[screenId]
			.show()
			.siblings('[data-co]')
			.hide();
	},
	refreshChild: function(eventName){
		this.children[eventName] && this.children[eventName].model.loadData && this.children[eventName].model.loadData();
	},
});

	return App;
});