define(['app', 'uicontrols', 'underscore', 'backbone'], function(App, $AppUI, _, backbone){

// @param {Array} cur
// @param {Array} def
function isDiff(cur, def){
	if(def && cur){
		var i = cur.length;

		if(typeof(cur) != typeof(def) || i != def.length){
			return true;
		}else{
			while(i-- > 0){
				if(cur[i] != def[i]){
					return true;
				}
			}
			return false;
		}
	}else{
		return false;
	}
}
// Attention! Remove model throut collection: settingsPage.lexCollection.remove('LinuxFirewall')
var LexModel = backbone.Model.extend({
	idAttribute: '_id',
	url: '/api/10.2/set_lexmap', // url for update model value
	destroy: function(){},
	// add/remove tag
	updateTag: function(tagName, tagLabel, remove, cb){
		var 	componentId = this.get('_parent') || this.id,
				data = {key: tagName};

		if(!remove){
			data.map = {option: []};
		}

		data = {
			option: [{
				key: componentId,
				map: {
					option: [{
						key: tagLabel,
						map: {
							option: [data]
						}
					}]
				}
			}]
		};

		App.xhr.post(this.url, data, function(d, r){
			if(r.sessionComplete) return;
			cb && cb(d);
		});
	},
	isMod: function(property, value){
		var value = value || this.get(property);
        var blockId = this.id;
        var parentObj;
        if (this.has('_parent')) {
            parentObj = this.collection.get(this.get('_parent'));
            blockId = parentObj.id;
        } else if (property == 'LogLevel'){
            parentObj = this.collection.get('Root');
            blockId = 'Root';
        }
        var referenceValue;
        if (parentObj) {
            referenceValue = parentObj.get(property);
        } else {
            var defaultValue = this.collection.defaultValues[blockId][property];
            var modifiedValue = this.collection.master[blockId] ? this.collection.master[blockId][property] : false;
            // If vaderetro is not installed. For some modules (MailD.RuleSetX, for example).
            if (modifiedValue && !defaultValue) {
                return true;
            }
            referenceValue = defaultValue;
        }

		if(Array.isArray(referenceValue)){
			return isDiff(value, referenceValue);
		}else{
			return false;
		}
	},
	// @param {Array} value
	change: function(property, value, cb){
		var 	data = {},
			 	component = this.get('_parent') || this.id,
				tag = this.get('_tag'),
				optionList = [{
					key: property,
					value: {
						item: value
					}
				}];

		if(tag){
			optionList = [{
				key: tag,
				map: {
					option: [{
						key: this.id.split('.').splice(-1)[0],
						map: {option: optionList}
					}]
				}
			}];
		}

		data.option = [{
			key: component,
			map: {
				option: optionList	
			}
		}];

    	App.xhr.post(this.url, data, function(d, r){
    		if(!d){ // maybe backend is turn off
    			window.location.href = App.helpers.generateBulkPage();
			}else if(r.sessionComplete){
				return;	
			} 

			// TODO recheck roolback engine
			if(d && Array.isArray(d.item)){
				var 	i = d.item.length,
						property,
						pos,
						modelId;

				while(i--){
					pos = d.item[i].option.lastIndexOf('.');

					if(pos > 0){
						modelId = d.item[i].option.substr(0, pos++);
						property = 	d.item[i].option.substr(pos);
						
						if(d.item[i].result != 'EC_OK'){ // model value not changed so restore value at interface
							this.changed = {};
							this.changed[property] = this.get(property);
							this.trigger('change', this);
						}else{
							this.set(property, value);
						}
					}
					cb && cb(d.item[i].result);
				}
			}
    	}.bind(this), 'json');
		return this.isMod(property, value)
	},
	groupChange: function(iniCollection, cb){
		var 	component = this.get('_parent') || this.id,
				tag = this.get('_tag'),
				optionList = [],
				property,
				defaults = this.collection.defaultValues[component];

		for(property in iniCollection){
			optionList.push({
				key: property,
				value: {
					item: iniCollection[property]
				}
			});
		}
		for(property in defaults){
			if(!iniCollection[property]){
				optionList.push({
					key: property,
					value: undefined,
				});		
			}
		}

		if(tag){
			optionList = [{
				key: tag,
				map: {
					option: [{
						key: this.id.split('.').splice(-1)[0],
						map: {option: optionList}
					}]
				}
			}];
		}

		App.xhr.post(this.url, {
			option: [{
				key: component,
				map: {
					option: optionList	
				}
			}]
		}, function(d, r){
    		if(!d){ // maybe backend is turn off
    			window.location.href = App.helpers.generateBulkPage();
			}else if(r.sessionComplete){
				return;	
			} 

			// TODO recheck roolback engine
			if(d && Array.isArray(d.item)){
				var 	i = d.item.length,
						errorList = [],
						property,
						pos,
						modelId;

				while(i-- > 0){
					pos = d.item[i].option.lastIndexOf('.'),
					rez = d.item[i].result;
					if (pos > 0){		
						modelId = d.item[i].option.substr(0, pos++);
						property = 	d.item[i].option.substr(pos);
						if(rez != 'EC_OK'){
							errorList.push(property);
							errorListJoin = errorList.join(', ');
						}
					}
					cb && cb(this);
				}
			}
    	}.bind(this), 'json');
	},
	defined: function(property){
		return this.attributes.hasOwnProperty(property);
	},
	generateIni: function(){
		var 	str = '',
				property,
				value;

		var rightOrder = [];
		for(var k in SCHEME[this.id]) {
            rightOrder = rightOrder.concat(Object.keys(SCHEME[this.id][k]));
        }
		var modified = [];
		for(property in this.attributes) {
			if(property[0] == '_') continue;
			value = this.attributes[property];
			if (this.isMod(property)) {
                modified[rightOrder.indexOf(property)] = {
                    key: property,
                    value: value
                };
			}
		}
		for (property in modified) {
            var item = modified[property];
            if (Array.isArray(item.value)) {
                var i = 0;
                while(i < item.value.length) {
                    // Turn off while fixing bug with escaping
                    str += item.key + ' = ' + item.value[i].replace('"', '\"') + '\n';
                    i++;
                }
            } else {
                str += item.key + ' = \n';
            }
        }
	
		return str;
	},
	// @raturn {Object} collection
	parseIni: function(rows){
		var  	pos,
				i = rows.length,
				property,
				value,
				collection = {};
		
		while(i-- > 0){
			pos = rows[i].indexOf('=');

			if(pos > 0){
				property = rows[i].substring(0, pos++).trim();
				value = rows[i].substr(pos).trim();

				if(value[0] == '"') value = value.slice(1, -1);
				if(!collection[property]) collection[property] = [];
				// Depricated
				if(value) collection[property].push(value); // collection[property].push(value.replace('\"', '"'));

			}
		}

		return collection;
	},
	tags: {
		LookupD: ['LDAP', 'AD'],
		SMBSpider: ['Share'],
		ClamD: ['Endpoint'],
		LinuxSpider: ['Site'],
	},
	iniMap: {
		Root: 'settings_root',
		LinuxSpider: 'APP_LINUX_SPIDER',
		MacSpider: 'APP_MAC_SPIDER',
		NSS: 'APP_LINUX_NSS',
		SMBSpider: 'APP_SMB_SPIDER',
		LinuxFirewall: 'APP_LINUX_FIREWALL',
		MacFirewall: 'APP_MAC_FIREWALL',
		ICAPD: 'APP_ICAPD',
		ClamD: 'APP_CLAMD',
		SNMPD: 'APP_SNMPD',
		ESAgent: 'APP_ESAGENT',
		HTTPD: 'APP_HTTPD',
		FileCheck: 'APP_FILE_CHECK',
		NetCheck: 'APP_NET_CHECK',
		ScanEngine: 'APP_SCAN_ENGINE',
		Update: 'APP_UPDATE',
		CloudD: 'APP_CLOUDD',
		GateD: 'APP_GATED',
		LookupD: 'APP_LOOKUPD',
		MailD: 'APP_MAILD',
		LinuxGUI: 'APP_LINUX_GUI',
		MacGUI: 'APP_MAC_GUI',
		Console: 'Console',
		Antispam : 'APP_ANTISPAM'
	},
});


var LexComponentView = backbone.View.extend({
	tagName: 'label',
	className: 'dwc_threat-group_item',
	template: 
		// '<span class="dwc_threat-group_label" data-i18="%1" tabindex="0">%2</span>' +
		'<a href="javascript:void(0)" class="dwc_threat-group_label" data-i18="%1" tabindex="0">%2</a>' +
		'<div class="dwc_threat-group_tags-btn"></div>' +
		'<div class="dwc_threat-group_tags"></div>',
	initialize: function(){
		this.render();
		var 	$groupBtn = this.el.querySelector('.dwc_threat-group_tags-btn'),
				$tags = this.el.querySelector('.dwc_threat-group_tags');
		
		if(!this.model.has('_tags') || (this.model.get('_tags') == '')){
			$groupBtn.style.display = 'none';			
		}else{
			$tags.innerHTML = this.renderTags();
		}

		this.listenTo(this.model, 'remove', function(){ // when model remove from collection
			this.remove();
		});
		this.listenTo(this.model, 'change:_active', function(m, isActive){
			this.el.classList[isActive ? 'add': 'remove']('__active');
		});
		this.listenTo(this.model, 'replaceView', function($parent){
			if(this.el.parentNode != $parent[0]){
				$parent.append(this.el);	
			}
		});
		this.listenTo(this.model, 'change:_tags', function(m, tags){
			if(tags.length > 0){
				$groupBtn.style.display = '';
				$tags.innerHTML = this.renderTags(tags);
			}else{
				$tags.innerHTML = '';
				$groupBtn.style.display = 'none';
			}
		});
		this.listenTo(this.model, 'change:_showTags', function(m, showTags){
			this.el.classList[showTags ? 'add': 'remove']('__open');
		});

		if(this.model.tags[this.model.id] != undefined){
			if(this.model.has('_tags')) {
				var $btn = document.createElement('div');
				$btn.className = 'dwc_threat-group_addtag-btn';
				$btn.setAttribute('tavindex', 0);
				this.el.appendChild($btn);
			}
		}
	},
	render: function(){
		var app = this.model.iniMap[this.model.get('_id')];

		this.el.innerHTML = this.template.
			replace('%1', app).
			replace('%2', App.helpers._(app));
	},
	events: {
		'click .dwc_threat-group_label': function(e){
			var model = e.target.dataset.model ? this.model.collection.get(e.target.dataset.model) : this.model;
			this.model.collection.activate(model.id);
		},
		// Add tag
		'click .dwc_threat-group_addtag-btn': function(e){
			e.stopPropagation();

			var 	_model = this.model, // get relative model from collection
					tagLabels = this.model.tags[this.model.id] || [],
					$label = e.target.parentNode,
					$container = e.target.previousElementSibling.previousElementSibling;

			new $AppUI.BasePopupView({
				title: App.helpers._('Add_set_of_parameters_eith_tag'),
				class: 'dec_wt-popup',
				content: 
				'<button data-co="close" class="dwc_popup-close"></button>' +
				'<form class="dwc-form" data-co="form">' +
					'<div class="dwc-form_offset1">' + App.helpers._('Specify_unique_tag') + '</div>' +
					'<div class="dwc_validation-input" data-warning="' + App.helpers._('pattern_Tag') + '">' +
						'<input type="text" class="dwc_validation-input_text" data-co="tag" pattern="[a-zA-Z0-9]+" required />' +
					'</div>' +
					(	tagLabels.length > 1 ? 
						tagLabels.map(function(tag){
							return '<label><input class="dwc-form_chbox" type="radio" value="' + tag + '" name="tag" checked/><span class="dwc-form_chbox-text">' + tag + '</span></label>';
						}).join('') : 
						('<input style="display:none;" class="dwc-form_chbox" type="radio" value="' + tagLabels[0] + '" name="tag" checked/>') 
					) +
					'<div class="dwc-form_btn-wrap">' +
						'<button class="dwc_btn __green" data-co="submit" type="submit" disabled>' + App.helpers._('add_btn') + '</button>' +
						'<button class="dwc_btn" data-co="cancel">' + App.helpers._('Cancel') + '</button>' +
					'</div>' +
				'</form>',
				_events: {
					'form submit': function(e){
						e.stopPropagation();
						e.preventDefault();

						var 	$checked = e.target.querySelector(':checked'),
								newTag = this.controls.tag.val().trim();
						
						_model.updateTag(newTag, $checked.value, false, function(){
							this.close();
						}.bind(this));
					},
					'tag input': function(e){
						this.controls.submit.prop('disabled', e.target.value.length == 0);
					},
					'tag invalid': function(e){
						e.preventDefault();
						e.target.parentNode.classList.add('__invalid');
						e.target.onfocus = function(e){
							e.target.parentNode.classList.remove('__invalid');
							e.target.onfocus = null;
						};
					},
					'cancel click': function(e){
						e.preventDefault();
						e.stopPropagation();
						this.close();
					},
					'close click':function(e){
						e.stopPropagation();
						this.close();
					},
				},
				onopen: function(view){
					setTimeout(function(){
						view.controls.tag.focus();	
					}, 0);
				}
			}).open();
		},
		'click .dwc_threat-group_tags-btn': function(e){
			e.stopPropagation();
			this.model.set('_showTags', !this.model.get('_showTags'));
		},
		// Del tag
		'click .dwc_threat-group_deltag-btn': function(e){
			e.stopPropagation();
			var 	$node = e.target.previousElementSibling,
					key = $node.dataset.model.split('.'),
					_model = this.model.collection.get($node.dataset.model);

			new $AppUI.BasePopupView({
				title: App.helpers._('Delete_parameter_set'),
				class: 'dec_wt-popup',
				content: 
				'<button data-co="close" class="dwc_popup-close"></button>' +
				'<form class="dwc-form" data-co="form">' +
					'<div class="dwc-form_offset1">' + App.helpers._('Delete_parameter_set_with_tag').replace('%s', '<span class="dwc-form_text" data-co="title"></span>') + '</div>' +
					'<div class="dwc-form_btn-wrap">' +
						'<button class="dwc_btn __green" type="submit">' + App.helpers._('remove_btn') + '</button>' +
						'<button class="dwc_btn" data-co="cancel">' + App.helpers._('Cancel') + '</button>' +
					'</div>' +
				'</form>' +
				'',
				onopen: function(view){
					view.controls.title.text(key[2]);
					view.controls.form.on('submit', function(e){
						e.preventDefault();
						_model.updateTag(key[2], key[1], true, function(d){
							view.close();	
						});
					});
					view.controls.cancel.on('click', function(e){
						e.stopPropagation();
						view.close();
					});
					view.controls.close.on('click', function(e){
						e.stopPropagation();
						view.close();
					});
				},
				onclose: function(view){}
			}).open();
		}
	},
	renderTags: function(tags){
		var 	tags = tags || this.model.get('_tags'),
				i = tags.length,
				buf;
		var 	src = '',
				template = '<label class="dwc_threat-group_item"><a href="javascript:void(0)" class="dwc_threat-group_label" data-model="%1">%2</a><span class="dwc_threat-group_deltag-btn"></span></label>';
		while(i-- > 0){
			buf = tags[i].split('.');
			src += template.replace('%1', tags[i]).replace('%2', buf[2]);	
		}
		return src;
	},
});

// PLAN for refactor- separate current LexCollection on SettingsCollection (reperesentation of get_lexmap) and SettingsPageModel (state of current page)

var LexCollection = backbone.Collection.extend({
	model: LexModel,
	url: '/api/10.2/get_lexmap',
	fetch: function(conf){
		App.xhr.get(this.url, {}, function(d, r){
			if(d && d.active){
				var 	defaults = {},
					 	list = d.hardcoded.option || [],
						i, innerList, buf, components;

				for(i = 0; i < list.length; i++){
	                innerList = list[i].map.option,
	                buf = defaults[list[i].key] = {};
	                
	                for(var j = 0; j < innerList.length; j++){
	                    if(innerList[j].value){
	                        buf[innerList[j].key] = innerList[j].value.item;
	                    }
	                }
				}		
				this.defaultValues = defaults;
				this.master = {};
				buf = this.collectMap(d.master.option || [], '');
				i = buf.length;

				while(i-- > 0){
					this.master[buf[i]._id] = buf[i];
				}

				components = this.collectMap(d.active.option || [], '');
				this.set(components);
				// TODO refactor
				conf && conf.active && this.activate(conf.active);
				this.trigger('lexReady', this);	
			} else if(r.sessionComplete){
				return;	
			} // else another troubles
		}.bind(this), 'json');
	},
	clearAll: function(conf){
		if(!(conf && conf.silent)){
			for(var i = 0; i < this.models.length; i++){
				this.trigger('remove', this.models[i], this);
			}
		}
		this.reset();
	},
	collectMap: function(list, compName, tagName, baseList){
		var 	finalList = baseList || [],
				i = list.length;
		
		while(i-- > 0){
			var 	innerList = list && list[i].map && list[i].map.option,
					collection = {
						_id: (compName ? compName + '.' : '') + (tagName ? tagName + '.' : '') + list[i].key,
					};

			if(compName) collection._parent = compName;
			if(tagName){
				collection._tag = tagName;
				collection._tagId = list[i].key;	
			} 

			var temp = this.collectSettings(innerList || [], collection, finalList);
			finalList.push(temp);
			
			if(temp._tags){
				var 	j = temp._tags.length,
						tagList = []

				while(j-- > 0){
					tagList.push(temp._tags[j]._id);
					finalList.push(temp._tags[j]);	
				}
				temp._tags = tagList;
			}
		}
		return finalList;
	},
	collectSettings: function(list, primarySettings, baseList){
		var 	i = list.length,
				key;

		while(i-- > 0){
			key = list[i].key;

			if(list[i].value){
				primarySettings[key] = list[i].value.item;
			}else if(list[i].map && list[i].map.option){
				if(!primarySettings._tags){
					primarySettings._tags = [];
				}
				this.collectMap(list[i].map.option, primarySettings._id, list[i].key, primarySettings._tags);
			}
		}
		return primarySettings;
	},
	activate: function(id){
		this.currentActive = id;
		var i = this.models.length;
		
		while(i-- > 0){
			this.models[i].set('_active', this.models[i].id == id);
		}
		this.trigger('activate', this, this.get(id));
	},
});

var ComponentSettingsView = backbone.View.extend({
	initialize: function(conf){
		this.render(conf.scheme, conf.model.id);
		this.listenTo(this.model, 'change', function(m){
			var 	property,
					$node,
					data,
					groupLabel,
					value;

			for(property in m.changed){
				value = m.changed[property];
				$node = this.el.querySelector('[data-settings=' + property + ']');

				for(groupLabel in conf.scheme){
					data = conf.scheme[groupLabel] && conf.scheme[groupLabel][property];

					if(data){
						break;
					}
				}
				if($node && data){
					$node.parentNode.parentNode.innerHTML = 
						'<div class="dwc_settings_cell"><h4 class="dwc_settings_title">' + property +'</h4>' +
							_.escape(App.i18n._('label_' + property) || App.i18n._('label_' + (m.get('_parent') || m.get('_id')) + '_' + property) || property) +
						'</div>' +
						this.renderControl(property, data);
				}
			}
		});
		this.listenTo(this.model, 'remove', function(){ // when model remove from collection
			this.remove();
		});
	},
	supplant: function(template, o){
		return template.replace(/{([^{}]*)}/g, function(a, b){
			var r = (b[0] == '=') ? $helpers.escape(o[b.substring(1)]) : o[b];
			return r != null ? r : '';
		});
	},
	renderControl: function(property, data){
		var 	config,
				template,
				typeList,
				value,
				type;

		config = {
			style: '',
			property: property,
		};
		typeList = data.type || data;
		type = Array.isArray(typeList) ? 'dropdown' : typeList;

		if(type[0] == '#'){
			data = type.substr(1);
			type = 'dropdown2';
		}
		value = this.model.get(property) || [];
		switch(type){
			case 'string':
				config.initValue = value[0] || '';
				config.pattern = data.pattern || '';
				config.emptyMes = data.emptyMes || App.helpers._('No_specified');
				config.warning = data.warning || '';
				if(!config.initValue) config.style += ' __empty';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.stringTemplate;
				break;
			case 'list':
				config.initValue = value.join(', ');
				config.emptyMes = data.emptyMes || App.helpers._('No_specified');
				if(!config.initValue) config.style += ' __empty';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.listTemplate;
				break;	
			case 'number':
				config.initValue = value[0] || '';
				config.emptyMes = data.emptyMes || App.helpers._('No_specified');
				if(data.postfix) config.postfix = data.postfix;
				if(!config.initValue) config.style += ' __minimal';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.visualInputTemplate;
				break;
			case 'bool':
				config.attr = this.model.get(property)[0] == 'Yes' ? ' checked="checked" ' : '';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.boolTemplate;
				break;
			case 'switcher':
				config.attr = this.model.get(property)[0] == 'On' ? ' checked="checked" ' : '';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.switcherTemplate;
				break;
			case 'dropdown':
				if(this.model.isMod(property)) config.style += ' __mod';
				config.selected = this.model.get(property)[0];
				config.initValue = _.escape(App.helpers._('item_' + config.selected));
				config.list = typeList.map(function(key){
					return key + ';' + _.escape(App.helpers._('item_' + key));
				}).join(';');
				template = this.dropDownTemplate;
				break;
			case 'dropdown2':
				if(this.model.isMod(property)) config.style += ' __mod';
				var 	list = _.escape(App.helpers._(data)),
						buf = list.split(';'),
						hash = {};

				for(var i = 0; i < buf.length; i += 2){
					hash[buf[i]] = buf[i + 1];
				}
				config.selected = this.model.get(property)[0];
				config.initValue = hash[config.selected];
				config.list = list;
				hash = buf = null;
				template = this.dropDownTemplate;
				break;
			case 'log':
				config.initValue = this.model.get(property)[0] || '';
				// config.pattern = '^\\/.*$|^Auto$|^Syslog\\:.+$|^Stderror\\:.+$';
				config.pattern = '^\/.*$|^Auto$|^Syslog\:.+$|^Stderror\:.+$';
				config.emptyMes = data.emptyMes || App.helpers._('No_specified');
				config.warning = 'pattern_Log';
				if(!config.initValue) config.style += ' __empty';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.stringTemplate;
				break;
			case 'divert':
				config.initValue = value[0] || '';
				if(!config.initValue) config.style += ' __empty';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.divertTemplate;
				break;
			case 'wizard':
				config.initValue = value[0] || '';
				if(!config.initValue) config.style += ' __empty';
				if(this.model.isMod(property)) config.style += ' __mod';
				template = this.wizardTemplate;
				break;	
		}

		if(config.emptyMes){
			config.emptyMes = _.escape(config.emptyMes);
		}
		if(config.initValue){
			config.initValue = _.escape(config.initValue);
		}

		return this.supplant(template, config);
	},
	render: function(scheme, compName){
		var 	html = '',
				groupHtml,
				component,
				group,
				groupLabel;

		for(groupLabel in scheme){
			group = scheme[groupLabel];
			groupHtml = '';
			for(var property in group){
				if(this.model.defined(property)){
					groupHtml += 
						'<div class="dwc_settings_row">' + 
							'<div class="dwc_settings_cell"><h4 class="dwc_settings_title">' + property +'</h4>' +
								_.escape(App.i18n._('label_' + property) || App.i18n._('label_' + compName + '_' + property) || property) +
							'</div>' +
							this.renderControl(property, group[property]) + 
						'</div>';
				}
			}

			if(groupHtml){
				html += 
				'<div class="dwc_settings_group">' +
					'<input type="checkbox" class="dwc_settings_group-checkbox" checked>' +
					'<h3 class="dwc_settings_group-title">' + App.helpers._(groupLabel) + '</h3>' +
					'<div class="dwc_settings_group-members">' + groupHtml + '</div>' +
				'</div>';
			}
		}

		this.el.innerHTML = html;
	},
	// Attention: divert Component is similar to wizardPopup - need to refactor this controls
	divertTemplate:
		'<div class="dwc_settings_cell">' +
			'<a href="javascript:void(0)" class="dwc_settings-label {style}" data-component="divert" data-settings="{property}">{initValue}</a>' +
		'</div>',
	wizardTemplate: // Configured at popup
		'<div class="dwc_settings_cell">' +
			'<a href="javascript:void(0)" class="dwc_settings-label {style}"  data-settings="{property}" data-component="wizard" data-settings="{property}">{initValue}</a>' +
		'</div>',	

	stringTemplate:  
		'<div class="dwc_settings_cell">' +
			'<a href="javascript:void(0)" class="{style}" data-component="singleline" data-settings="{property}" data-value="{initValue}" data-empty="{emptyMes}" data-title="{title}" data-pattern="{pattern}" data-warning="{warning}"></a>' +
		'</div>',
	visualInputTemplate:
		'<div class="dwc_settings_cell">' + 
			'<span class="dwc_visual-input {style}" data-mes="{emptyMes}" data-settings="{property}" data-component="visualinput" contenteditable data-postfix="{postfix}">{initValue}</span>' +
		'</div>',
	dropDownTemplate: 
		'<div class="dwc_settings_cell">' + 
			'<a href="javascript:void(0)" class="dwc_dropdown {style}" data-component="dropdown" data-value="{initValue}" data-id="{selected}" data-settings="{property}" data-list="{list}"></a>' +
		'</div>',
	boolTemplate:
		'<label class="dwc_settings_cell dwc_chbox">' + 
			'<input type="checkbox" data-component="switcher" {attr} data-settings="{property}" data-true="Yes" data-false="No" class="{style}"><span class="dwc_chbox-icn"></span>' +
		'</label>',
	switcherTemplate: 
		'<label class="dwc_settings_cell dwc_chbox">' +
			'<input type="checkbox" data-component="turnswitcher" {attr} data-settings="{property}" data-true="On" data-false="Off" class="{style}"><span class="dwc_chbox-icn"></span>' +
		'</label>',
	listTemplate:
		'<div class="dwc_settings_cell">' + 
			'<a href="javascript:void(0)" class="{style}" data-component="customlist" data-settings="{property}" data-value="{initValue}" data-title="{title}" data-empty="{emptyMes}"></a>' +
		'</div>',
	events: {
		dropdownselected: function(e){
			var 	settingsKey = e.target.dataset.settings,
					value = [e.target.dataset.id];

			if(settingsKey){
				if(this.model.change(settingsKey, value)){
					e.target.classList.add('__mod');
				}else{
					e.target.classList.remove('__mod');
				}
			}
		},
		visualinputchange: function(e){
			var 	settingsKey = e.target.dataset.settings,
					value = e.target.textContent;

			if(settingsKey){
				var num = parseInt(value) || 0;
				e.target.classList[ num > 0 ? 'remove' : 'add' ]('__minimal');

				if(this.model.change(settingsKey, [value])){
					e.target.classList.add('__mod');
				}else{
					e.target.classList.remove('__mod');
				}					
			}
		},
		'change': function(e){ // bool switcher
			var settingsKey = e.target.dataset.settings;

			if(settingsKey){
				var 	value = e.target.checked ? e.target.dataset.true : e.target.dataset.false;

				e.target.classList[this.model.change(settingsKey, [value]) ? 'add' : 'remove']('__mod');
			}	
		},
		'click .dwc_settings_group-title': function(e){
			var 	$checkbox = e.target.previousElementSibling;
			$checkbox.checked = !$checkbox.checked;
		},
		'click [data-component=customlist]': function(e){
			var 	$target = e.target,
					self = this,
					settingsKey = $target.dataset.settings,
					path = this.model.get(settingsKey) || [];

			new $AppUI.BasePopupView({
				title: _.escape($target.dataset.settings),
				class: 'dec_wt-popup',
				content: 
				'<button data-co="close" class="dwc_popup-close"></button>' +
				'<form class="dwc-form" data-co="form">' +
					'<div class="dwc-form_validation-textarea" data-warning="">' + // for warning message
						'<textarea class="dwc-form_validation-textarea_text" data-co="path" autofocus></textarea>'+
					'</div>' +
					'<div class="dwc-form_btn-wrap">' +
						'<button class="dwc_btn __green" type="submit">' + App.helpers._('save_btn') + '</button>' +
						'<button class="dwc_btn" data-co="cancel">' + App.helpers._('Cancel') +'</button>' +
					'</div>' +
				'</form>' +
				'',
				onopen: function(view){
					view.controls.path.val(path.join('\n'));

					view.controls.form.on('submit', function(e){		
						e.preventDefault();
						var  	list = view.controls.path.val().split('\n').map(function(s){return s.trim();})

						if(self.model.change(settingsKey, list, function(ec){
							if(ec != 'EC_OK') {
								view.controls.path.parent().attr('data-warning', App.helpers._(ec));
								view.controls.path.parent().addClass('__invalid');
							} else {
								view.close();
							}
						})){
							$target.classList.add('__mod');
						}else{
							$target.classList.remove('__mod');
						}
					});
					view.controls.cancel.on('click', function(e){
						view.close();
					});
					view.controls.path.on('focus', function(e){
						e.target.parentNode.classList.remove('__invalid');
					});
					setTimeout(function(){
						view.controls.path.focus();	
					}, 100);
					view.controls.close.on('click', function(e){
						e.stopPropagation();
						e.preventDefault();
						view.close();
					});
				},
				onclose: function(view){}
			}).open();
		},
		'click [data-component=singleline]': function(e){
			var 	$target = e.target,
					settingsKey = $target.dataset.settings,
					path = $target.dataset.value,
					self = this;

			new $AppUI.BasePopupView({
				title: _.escape($target.dataset.settings),
				class: 'dec_wt-popup',
				content: 
				'<button data-co="close" class="dwc_popup-close"></button>' +
				'<form class="dwc-form" data-co="form">' +
					'<div class="dwc_validation-input" data-warning="">' + // for warning message
						'<input type="text" class="dwc_validation-input_text" data-co="path"/>' +
					'</div>' +
					'<div class="dwc-form_btn-wrap">' +
						'<button class="dwc_btn __green" type="submit">' + App.helpers._('save_btn') + '</button>' +
						'<button class="dwc_btn" data-co="cancel">' + App.helpers._('Cancel') + '</button>' +
					'</div>' +
				'</form>' +
				'',
				_events: {
					// DEPRICATED 
					// 'form submit': function(e){
					// 	e.preventDefault();
					// 	var newPath = this.controls.path.val().trim();

					// 	$target.classList[newPath ? 'remove' : 'add']('__empty');
					// 	$target.classList[self.model.isMod(settingsKey, newPath) ? 'add' : 'remove']('__mod');
					// 	$target.dataset.value = newPath;
					// 	$target.setAttribute('data-value', newPath);

					// 	self.model.change(settingsKey, [newPath], settingsKey == 'ListenAddress' && function(){
					// 		window.location.href = 'https://' + newPath + '/';
					// 	});
					// 	this.close();
					// },
					'cancel click': function(e){
						e.stopPropagation();
						this.close();
					},
					'close click': function(e){
						e.stopPropagation();
						this.close();
					},
				},
				onopen: function(view){
					// DEPRICATED (case sensitive)
					// if($target.dataset.pattern){ // One variant of form validation
					// 	view.controls.path.attr('pattern', $target.dataset.pattern);
					// 	view.controls.path.attr('required', true);
					// 	view.controls.path.on('invalid', function(e){
					// 		e.preventDefault();
					// 		e.target.parentNode.classList.add('__invalid');
					// 		e.target.onfocus = function(e){
					// 			e.target.parentNode.classList.remove('__invalid');
					// 			e.target.onfocus = null;
					// 		};
					// 	});
					// 	view.controls.path.parent().attr('data-warning', App.helpers._('pattern_' + settingsKey));
					// }

					if($target.dataset.pattern){
						var 	_pattern = new RegExp($target.dataset.pattern, 'i');

						view.controls.path.
							attr('required', true).
							parent().attr('data-warning', App.helpers._($target.dataset.warning || 'pattern_' + settingsKey));
						view.controls.form.on('submit', function(e){
							e.preventDefault();

							var 	value = view.controls.path.val().trim();

							if(_pattern.test(value)){
								$target.classList[value ? 'remove' : 'add']('__empty');
								$target.classList[self.model.isMod(settingsKey, value) ? 'add' : 'remove']('__mod');
								$target.dataset.value = value;
								$target.setAttribute('data-value', value);
								self.model.change(settingsKey, [value], settingsKey == 'WebConsoleAddress' && function(){
									window.location.href = 'https://' + value + '/';
								});
								view.close();
							}else{
								view.controls.path.parent().addClass('__invalid');
							}
						});
						view.controls.path.on('focus', function(e){
							e.target.parentNode.classList.remove('__invalid');
						});
					}else{
						view.controls.form.on('submit', function(e){
							e.preventDefault();
							var 	value = view.controls.path.val().trim();

							$target.classList[value ? 'remove' : 'add']('__empty');
							$target.classList[self.model.isMod(settingsKey, value) ? 'add' : 'remove']('__mod');
							$target.dataset.value = value;
							$target.setAttribute('data-value', value);
							self.model.change(settingsKey, [value], settingsKey == 'WebConsoleAddress' && function(){
								window.location.href = 'https://' + value + '/';
							});
							view.close();
						});
					}

					view.controls.path.val(path);
					setTimeout(function(){
						view.controls.path.focus();	
					}, 100);
				},
			}).open();
		},
		'click [data-component=divert]': function(e){
			var 	property = e.target.dataset.settings,
					self = this;

			require(['divert.popup'], function(popup){
				popup(property, this.model.get(property), function(settingLine){
					self.model.change(property, [settingLine]);
				});
			}.bind(this));
		},
		'click [data-component=wizard]': function(e){
			var 	property = e.target.dataset.settings,
					setting = e.target.dataset.settings,
					self = this;

			if(setting == 'RepackPassword'){
				require(['repackpass.popup'], function(popup){
					popup(property, this.model.get(property), function(settingLine){
						self.model.change(property, [settingLine]);
					});
				}.bind(this));
			}
		}
	},
});

	App.view.settings = App.view.base.extend({
		className: 'dwc_dcolumn-wrap', 
		template: 
		'<div class="dwc_dcolumn-panel">'+
			'<div class="dwc_set_panel-wrap" data-co="left">' +
				'<h2 data-i18="Settings" class="dwc_set_header">' + App.helpers._('Settings') + '</h2>' +
				'<div class="dwc_set_block" data-co="root-componets"></div>' +
				'<div class="dwc_set_block" data-co="primary-componets"></div>' +
				'<div data-co="additional-componets"></div>' +
				'<div data-co="hidden-componets" style="display:none;"></div>' +
				//----------------------------------------------------------
				'<div class="dwc_threat-group_item" data-hardcoded="mode"><a href="javascript:void(0)" class="dwc_threat-group_label" data-i18="cpm_mode" tabindex="0">' + App.helpers._('cpm_mode')+ '</a></div>' +
				'<div class="dwc_threat-group_item" data-hardcoded="console"><a href="javascript:void(0)" class="dwc_threat-group_label" data-i18="Console" tabindex="0">' + App.helpers._('Console') + '</a></div>' +
				//----------------------------------------------------------
			'</div>' +
		'</div>' +
		'<div class="dwc_dcolumn-body">' +
			'<div class="dwc_set_body" data-co="body">' +		
				'<div class="dwc_thr_header-wrap">' +
					'<h2 class="dwc_set_comp" data-co="set-title"></h2>' +
					'<div class="dwc_thr_search-wrap" data-co="search-wrap">' +
						'<input type="text" class="dwc_search-input dwc-form_base-input" data-co="search-input" />' +
						'<div class="dwc_search-btn" data-co="search-btn"></div>' +
					'</div>' +
					'<div class="dwc_thr_filters">' +
						'<span class="dwc_set_group-title" data-co="set-sub-title"></span>' +
					'</div>' +
				'</div>' +
				'<div class="dwc_thr_header-wrap" data-co="filters">' +
					'<label class="dwc_set_radio">' +
						'<input class="dwc_set_radio-checkbox" type="radio" name="switchmod" data-co="switch-all" value="all" checked>' + 
						'<span class="dwc_set_radio-label" data-i18="All" >' + App.helpers._('All') + '</span>' +
					'</label>' +
					'<label class="dwc_set_radio">' +
						'<input class="dwc_set_radio-checkbox" type="radio" name="switchmod" value="modified">' + 
						'<span class="dwc_set_radio-label" data-i18="Changed" >' + App.helpers._('Changed') + '</span>' +
					'</label>' +
					'<label class="dwc_set_radio">' +
						'<input class="dwc_set_radio-checkbox" type="radio" name="switchmod" data-co="switch-ini" value="ini">' + 
						'<span class="dwc_set_radio-label" data-i18="Ini_editor">' + App.helpers._('Ini_editor') + '</span>' +
					'</label>' +
				'</div>' +
				'<div data-co="set-list" class="clearfix dwc_set_list"></div>' +
				'<div class="dwc_set_no-results" data-co="no-results" style="display:none;"></div>' +
				'<form class="dwc_set_ini-wrap" data-co="ini-form" style="display:none;">' +
					'<div class="dwc_set_ini-desc" data-co="ini-desc" data-i18="Ini_editor_description">' + App.helpers._('Ini_editor_description') + '</div>' +
					'<div class="dwc_set_ini-textbox_validation" data-warning="">' + 
						'<textarea data-co="ini-text" class="dwc_set_ini-textbox"></textarea>' +
					'</div>' +
					'<div class="dwc_set_ini-btns">' +
						'<span style="display:none;" data-co="ini-mes" data-i18="The_changes_are_not_yet_saved">' + App.helpers._('The_changes_are_not_yet_saved') + '</span>'+
						'<button type="submit" class="dwc_btn __green" disabled data-co="ini-submit" data-i18="save_btn">' + App.helpers._('save_btn') +'</button>' +
						'<button type="reset" class="dwc_btn" disabled data-co="ini-reset" data-i18="reset_btn">' + App.helpers._('reset_btn') + '</button>' +
					'</div>' +
				'</form>' +
				'<div class="dwc_set_preloader" data-co="preloader"set-list></div>' +
			'</div>' +
			'<div class="dwc_set_body" data-co="space" style="display:none;"></div>'+
		'</div>' +
		'',
		disableIniButtons: function(status){
			this.controls.iniSubmit.prop('disabled', status);
			this.controls.iniReset.prop('disabled', status);
			this.controls.iniMes[status ? 'hide' : 'show']();
		},
		events: {
			// New! chit code
			'click .dwc_threat-group_item': function(e){
				var 	$target = e.target.parentNode,
						$active = this.controls.left.find('.dwc_threat-group_item.__active'),
						i = $active.length,
						confPage = $target.dataset.hardcoded;

				while(i-- > 0){
					if($active[i] != $target){
						$active[i].classList.remove('__active');
					}
				}
				$target.classList.add('__active');

				if(confPage){ // Open hardcoded page
					require(['settings_' + confPage], function(Constructor){
						this._setView && this._setView.remove();
						this._setView = new Constructor({});
						this.controls.space.empty().append(this._setView.el).show();
						this.controls.body.hide();
						App.helpers.localize(this.el);
					}.bind(this), function(err){ 
						App.user.checkAutorization();
					});
				}
			},
			'submit [data-co=ini-form]': function(e){		
				e.preventDefault();

				if(this._setView && this._setView.model){
					this.controls.iniForm.addClass('__load');
					var 	iniCollection = this._setView.model.parseIni(this.controls.iniText.val().split('\n'));

					this._setView.model.groupChange(iniCollection, function(model){
						// todo get ini and paste at textarea
						this.controls.iniText.val(model.generateIni());
						this.controls.iniForm.removeClass('__load');
						this.disableIniButtons(true);
						if(rez != 'EC_OK'){
							this.controls.iniText.parent().attr('data-warning', (App.helpers._('Invalid_parameter_values')) + ':' + ' ' + errorListJoin);
							this.controls.iniText.parent().addClass('__invalid');
						}
					}.bind(this));
				}
			},
			'reset [data-co=ini-form]': function(e){
				e.preventDefault();
				this.controls.iniText.val(this._setView ? this._setView.model.generateIni() : '');
				this.disableIniButtons(true);
			},
			'input [data-co=ini-text]': function(e){
				this.disableIniButtons(false);
			},
			// Function is turn Off
			// 'change [data-settings=Start]': function(e){ // confirmation before turn off component
			// 	if(!e.target.checked){
			// 		e.target.checked = true;
			// 		e.stopPropagation();
			// 		e.preventDefault();	

			// 		var 	_view = this,
			// 				_input = e.target;

			// 		App.helpers.confirm(App.helpers._('Turn_off_the_component'), App.helpers._('Component_will_cease_to'), function(){
			// 			_input.checked = false;
			// 			_view.events.change.call(_view, {target: _input});
			// 		}.bind(this), function(){});
			// 	}
			// },
			'focus [data-co=ini-form]' : function(e){
				this.controls.iniText.parent().removeClass('__invalid');
			},
			'input .dwc_search-input': function(e){
				this.search(e.target.value.toLowerCase());
			},
			'click .dwc_threat-group_label':  function(e){
				this.controls.iniText.parent().removeClass('__invalid');
			},
			'click .dwc_clear-btn': function(e){
				this.switchList(0);
			},
			'change [name=switchmod]': function(e){
				if(e.target.value == 'modified'){
					this.switchList(1);
				}else if(e.target.value == 'all'){
					this.switchList(0);
				}else{
					this.switchList(2);
				}
			}
		},
		search: _.debounce(function(val){
			if(val){
				this.switchList(3, val);	
			}else{
				this.switchList(0);	
			}
		}, 200),
		initialize: function(conf){
			App.view.base.prototype.initialize.call(this);

			this.listenTo(App.bus, 'refresh', function(e){
				if(e.event == 'lexmap'){
					this.lexCollection.fetch();
				}
			});

			if(App.collection.installed.length == 0){
				App.collection.installed.fetch({reset: true});
			}
			this.on('remove', function(){});
			
			this.lexCollection = new LexCollection();
			// TODO refactor that code 
			this.lexCollection.fetch({
				active: conf.page || 'Root'
			});
			
			var activeSet = conf.page || 'Root';

		
			this.listenTo(this.lexCollection, 'add', function(model, collection){
				if(!model.has('_parent')){
					var itemView = new LexComponentView({
						model: model
					});
					this.controls.primaryComponets.append(itemView.el);	
				}
			})
			// Attention: second arguments can have different type if change collection or model
			this.listenTo(this.lexCollection, 'change', function(model){
				// when ini editor open and model has changed
				if(model.id == model.collection.currentActive && this.controls.switchIni.prop('checked')){
					this.controls.iniText.val(model.generateIni());
				}
			});
			this.listenTo(this.lexCollection, 'activate', function(collection, model){
				var 	resId = model.has('_parent') ? model.get('_parent') + '_' + model.get('_tag') : model.id,
						i18nKey = model.iniMap[model.get('_parent') || model.id],
						subTitle = !model.has('_parent') ? ('[' + model.id + ']') : App.helpers._(resId).replace(/%1/g, model.get('_tagId'));

				this.controls.space.hide();
				this.controls.body.show();
				this.controls.setTitle.
					text(App.helpers._(i18nKey)).
					attr('data-i18', i18nKey);
				this.controls.setSubTitle.text(subTitle);
				this.controls.setList.show();
				this.switchList(0);
				this.disableIniButtons(true);
				this._setView && this._setView.remove();
				this._setView = new ComponentSettingsView({
					model: model,
					scheme: SCHEME[resId]
				});
				this.controls.setList.empty();
				this.controls.setList.append(this._setView.el);
			});

			// Refresh component list at independent threat
			var buildComponentList = function(lexCollection, installed){
				var 	//iniMap = App.collection.installed.getIniMap(), // can be null
						iniMap = installed.getIniMap(),
					 	i = lexCollection.models.length,
					 	parent, id, app;

				if(iniMap){
					while(i-- > 0){
						id = lexCollection.models[i].id;

						if(app = iniMap[id]){
							if(id == 'Root'){
								parent = this.controls.rootComponets;
							}else if(id == 'ClamD' || id == 'ICAPD' || id == 'SMBSpider' || id == 'MacSpider' || id == 'MacFirewall' || id == 'LinuxSpider' || id == 'NSS' || id == 'LinuxFirewall'){
								parent = this.controls.primaryComponets;
							}else{
								parent = this.controls.additionalComponets;	
							}
						}else{
							if(id == 'Console'){
								parent = this.controls.left;
							}else{
								parent = this.controls.hiddenComponets;	
							}
						}
						lexCollection.models[i].trigger('replaceView', parent); 	
					}
				}
			}.bind(this);

			this.listenTo(App.collection.installed, 'reset', function(installed){
				buildComponentList(this.lexCollection, installed);
			});
			this.listenTo(this.lexCollection, 'lexReady', function(collection){
				// hardcore console settings model
				buildComponentList(collection, App.collection.installed);
			});
		},
		// @param {Int} modeId
		// 0: All
		// 	1) navigate()
		// 	2) radio[all]
		// 	3) close: click
		// 1: changed
		// 	1) radio[change]
		// 2: ini
		// 	1) radio[ini]
		// 3: search
		// 	1) input at search form 
		switchList: function(modeId, val){
			var 	resultCount = 0;
			
			if(modeId != 3){
				// Clean search bar!
				this.controls.searchInput.val('');
				this.controls.searchBtn.removeClass('dwc_clear-btn');
			}

			if(modeId < 2){
				this.controls.searchWrap.show();
			}

			if(modeId == 1){ // changed or insert empty string at search line
				this.controls.setList.show();
				this.controls.iniForm.hide();
				var 	$rowList = this.controls.setList.find('.dwc_settings_row'),
						$modList = this.controls.setList.find('.__mod');
				
				$rowList.hide().parent().parent().hide(); // init state
				$modList.each(function(index, $mod){
					$($mod).closest('.dwc_settings_row').show().parent().prev().prev().prop('checked', true).parent().show();
					resultCount++;
				});

				if(resultCount > 0){
					this.controls.noResults.hide();
				}else{
					this.controls.noResults.show().text(App.helpers._('All_params_are_the_same_as_default'));
				}
			}else if(modeId == 0){ // All
				this.controls.switchAll.prop('checked', true);
				this.controls.setList.show();
				this.controls.iniForm.hide();
				this.controls.noResults.hide();
				var 	$rowList = this.controls.setList.find('.dwc_settings_row');
				$rowList.show().parent().parent().show();
			}else if(modeId == 2){ // INI
				this.controls.searchWrap.hide();
				this.controls.setList.hide();
				this.controls.iniForm.show();
				this.controls.noResults.hide();
				var ini = this._setView && this._setView.model.generateIni ? this._setView.model.generateIni() : '';

				this.controls.iniText.val(ini);
			}else if(modeId == 3){ // search
				var 	val = val.toLowerCase(),
						$rowList = this.controls.setList.find('.dwc_settings_row');

				this.controls.setList.show();
				this.controls.iniForm.hide(); // At search hide ini block

				if(val){
					this.controls.searchBtn.addClass('dwc_clear-btn');
					$rowList.hide().parent().parent().hide(); // init state

					// check textContent:
					$rowList.each(function(index, $row){
						if($row.textContent.toLowerCase().indexOf(val) != -1){
							$($row).show().parent().prev().prev().prop('checked', true).parent().show();
							resultCount++;
						}
					});
					
					var $valueList = this.controls.setList.find('[data-value]');
					$valueList.each(function(index, $item){
						if($item.dataset.value.toLowerCase().indexOf(val) != -1){
							$($item).show().parent().prev().prev().prop('checked', true).parent().show();
							resultCount++;
						}
					});

					if(resultCount > 0){
						this.controls.noResults.hide();
					}else{
						this.controls.noResults.show().text(App.helpers._('No_results'));
					}
					// this.controls.switchAll.prop('checked', true);
					this.$el.find('[name=switchmod]').prop('checked', false);
				}
			}
		},
		remove: function(){
			if(this.lexCollection){
				this.lexCollection.totalDestroy();
			}
			this._setView && this._setView.remove();
			this._setView = null;
			App.view.base.prototype.remove.call(this);
		}
	});

// bool - yes/no
// switcher - on/off
var SCHEME = {
	LookupD: {
		Sect_General: {
			LogLevel: {
				type: ['Debug', 'Info', 'Notice', 'Warning', 'Error']
			},
			Log: 'log',
			RunAsUser: 'string',
			IdleTimeLimit: 'number',
			DebugLibldap: 'bool',
		},
		Sect_Security: {
			LdapCheckCertificate: '#items_LdapCheckCertificate',
			LdapCaCertificatePath: 'string',
		},
		Sect_Debug: {
			DebugLibldap: 'bool',
		}
	},
	LookupD_Ldap: {
		Sect_General: {
			Url: 'string',
			BindDn: 'string',
			BindPassword: 'string',
			ChaseReferrals: 'bool'
		}
	},	
	LookupD_Ad: {
		Sect_General: {
			Host: 'string',
			Port: 'number',
			Dn: 'string',
			User: 'string',
			Password: 'string',
			ChaseReferrals: 'bool',
			UseSSL: 'bool'
		}
	},
	LinuxSpider_Site: {
		Sect_General: {
			Path: 'string',
			Enable: 'bool',
		},
		Sect_Exclusions: {
			ExcludedPath: 'list',
		},
		Sect_Scansettings: {
			OnKnownVirus: ['Cure', 'Quarantine', 'Delete'],
			OnIncurable: ['Quarantine', 'Delete'],
			OnSuspicious: ['Report', 'Quarantine', 'Delete'],
			OnAdware: ['Report', 'Quarantine', 'Delete'],
			OnDialers: ['Report', 'Quarantine', 'Delete'],
			OnJokes: ['Report', 'Quarantine', 'Delete'],
			OnRiskware: ['Report', 'Quarantine', 'Delete'],
			OnHacktools: ['Report', 'Quarantine', 'Delete'],
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			PackerMaxLevel: 'number',
			ArchiveMaxLevel: 'number',
			MailMaxLevel: 'number',
			ContainerMaxLevel: 'number',
			MaxCompressionRatio: 'number'
		}
	},
	Antispam: {
		Sect_General: {
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
			IdleTimeLimit: 'number',
			RunAsUser: 'string',
			FixedSocketPath: 'string',
		}	
    },
	MailD: {
		Sect_General: {
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
			DnsResolverConfPath: 'string', 
			// RepackPassword: {
			// 	type: 'string', 
			// 	pattern: '^Plain\\([^\\)]+\\)$|^Secured\\([^\\)]+\\)$|^None$'
			// },
			RepackPassword: 'wizard',
			RunAsUser: 'string',
			FixedSocketPath: 'string',
		},
		Sect_Templates: {
			TemplatesDir: 'string',
			TemplateContacts: 'string',
			ReportLanguages: 'list',
		},
		Sect_MilterConections: { 
			MilterScanTimeout: 'number', 
			MilterHeuristicAnalysis: 'switcher',
			MilterPackerMaxLevel: 'number',
			MilterArchiveMaxLevel: 'number',
			MilterMailMaxLevel: 'number',
			MilterContainerMaxLevel: 'number',
			MilterMaxCompressionRatio: 'number',
			MilterSocket: 'string',
			MilterBlockUnchecked: 'bool',
			MilterRuleSet0: 'list',
			MilterRuleSet1: 'list',
			MilterRuleSet2: 'list',
			MilterRuleSet3: 'list',
			MilterRuleSet4: 'list',
			MilterRuleSet5: 'list',
			MilterRuleSet6: 'list',
			MilterRuleSet7: 'list',
			MilterRuleSet8: 'list',
		},
		Sect_SpamdConnections: { 
			SpamdScanTimeout: 'number', 
			SpamdHeuristicAnalysis: 'switcher',
			SpamdPackerMaxLevel: 'number', 
			SpamdArchiveMaxLevel: 'number', 
			SpamdMailMaxLevel: 'number', 
			SpamdContainerMaxLevel: 'number', 
			SpamdMaxCompressionRatio: 'number', 
			SpamdSocket: 'string', 
			SpamdBlockUnchecked: 'bool', 
			SpamdRuleSet0: 'list',
			SpamdRuleSet1: 'list',
			SpamdRuleSet2: 'list', 
			SpamdRuleSet3: 'list',
			SpamdRuleSet4: 'list',
			SpamdRuleSet5: 'list',
			SpamdRuleSet6: 'list',
			SpamdRuleSet7: 'list',
			SpamdRuleSet8: 'list',
		},
		Sect_RSpamdConnections: {
			RspamdScanTimeout: 'number', 
			RspamdHeuristicAnalysis: 'switcher',
			RspamdPackerMaxLevel: 'number', 
			RspamdArchiveMaxLevel: 'number', 
			RspamdMailMaxLevel: 'number', 
			RspamdContainerMaxLevel: 'number', 
			RspamdMaxCompressionRatio: 'number', 
			RspamdSocket: 'string', 
			RspamdBlockUnchecked: 'bool',
			RspamdRuleSet0: 'list',
			RspamdRuleSet1: 'list',
			RspamdRuleSet2: 'list',
			RspamdRuleSet3: 'list',
			RspamdRuleSet4: 'list',
			RspamdRuleSet5: 'list',
			RspamdRuleSet6: 'list',
			RspamdRuleSet7: 'list',
			RspamdRuleSet8: 'list',
		},
		// DEPRICATED ?
		// Sect_Mailfilter: {
		// 	SmtpBlockingPolicy: '#items_SmtpBlockingPolicy',
		// 	Pop3BlockingPolicy: '#items_BlockingPolicy',
		// 	ImapBlockingPolicy: '#items_BlockingPolicy',
		// },
		// Sect_Scansettings: {
		// 	ScanTimeout: 'number',
		// 	HeuristicAnalysis: 'switcher',
		// 	ArchiveMaxLevel: 'number',
		// 	MailMaxLevel: 'number',
		// 	PackerMaxLevel: 'number',
		// 	ContainerMaxLevel: 'number',
		// 	MaxCompressionRatio: 'number',
		// },
		// Sect_MTA: {
		// 	FixedSocketPath: 'string',
		// 	TemplatesDir: 'string',
		// 	MilterSocket: 'string',
		// 	SpamdSocket: 'string',
		// 	RspamdSocket: 'string',
		// 	MilterRuleSet: 'list',
		// },
		
	},
	NetCheck: {
		Sect_General: {
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
			LoadBalanceUseSsl: 'bool',
			LoadBalanceServerSocket: 'string',
			LoadBalanceAllowFrom: 'list',
			LoadBalanceSourceAddress: 'string',
			LoadBalanceTo: 'list',
		},
		Sect_SSL: {
			LoadBalanceSslCertificate: 'string',
			LoadBalanceSslKey: 'string',
			LoadBalanceSslCa: 'string',
		},
		Sect_Advanced: {
			FixedSocketPath: 'string',
			IdleTimeLimit: 'number',
			RunAsUser: 'string',
			SpoolDir: 'string',
			LoadBalanceStatusInterval: 'number',
			LocalScanPreference: 'number',
		}
	},
	ScanEngine: {
		Sect_General: {
			MaxForks: 'number',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Advanced: {
			FixedSocketPath: 'string',
			MaxForksPerFile: 'number',
			IdleTimeLimit: 'number',
			WatchdogInterval: 'number',
		},
	},
	SNMPD: {
		Sect_General: {
			Start: 'bool',
			RunAsUser: 'string',
			ListenAddress: 'string',
			SnmpVersion: ['V2c', 'V3'],
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Advanced: {
			TrapReceiver: 'list',
			V2cCommunity: 'string',
			V3EngineId: 'string',
			V3UserName: 'string',
			V3Auth: {
				type: 'string',
				pattern: '^SHA\\([^\\)]+\\)$|^MD5\\([^\\)]+\\)$|^None$'
			},
			V3Privacy: {
				type: 'string',
				pattern: '^DES\\([^\\)]+\\)$|^AES128\\([^\\)]+\\)$|^None$'
			}
		},
	},
	LinuxSpider: {
		Sect_General: {
			Start: 'bool',
			Mode: ['Auto', 'Fanotify', 'LKM'], 
			KextPath: 'string',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Scansettings: {
			OnKnownVirus: ['Cure', 'Quarantine', 'Delete'],
			OnIncurable: ['Quarantine', 'Delete'],
			OnSuspicious: ['Report', 'Quarantine', 'Delete'],
			OnAdware: ['Report', 'Quarantine', 'Delete'],
			OnDialers: ['Report', 'Quarantine', 'Delete'],
			OnJokes: ['Report', 'Quarantine', 'Delete'],
			OnRiskware: ['Report', 'Quarantine', 'Delete'],
			OnHacktools: ['Report', 'Quarantine', 'Delete'],
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			ArchiveMaxLevel: 'number',
			MailMaxLevel: 'number',
			PackerMaxLevel: 'number',
			ContainerMaxLevel: 'number',
			MaxCompressionRatio: 'number',
		},
		Sect_Exclusions: {
			IncludedPath: 'list',
			ExcludedPath: 'list',
			ExcludedProc: 'list',
		},
		Sect_Debug: {
			DebugFiles: 'bool',
			DebugAccess: 'bool'
		}
	},
	NSS: {
		Sect_General: {
			Start: 'bool',
			NssVolumesMountDir: 'string',
			ProtectedVolumes: 'list',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Scansettings: {
			OnKnownVirus: ['Cure', 'Quarantine', 'Delete'],
			OnIncurable: ['Quarantine', 'Delete'],
			OnSuspicious: ['Report', 'Quarantine', 'Delete'],
			OnAdware: ['Report', 'Quarantine', 'Delete'],
			OnDialers: ['Report', 'Quarantine', 'Delete'],
			OnJokes: ['Report', 'Quarantine', 'Delete'],
			OnRiskware: ['Report', 'Quarantine', 'Delete'],
			OnHacktools: ['Report', 'Quarantine', 'Delete'],
			OnError: ['Report', 'Quarantine', 'Delete'],
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			ArchiveMaxLevel: 'number',
			MailMaxLevel: 'number',
			PackerMaxLevel: 'number',
			ContainerMaxLevel: 'number',
			MaxCompressionRatio: 'number',
		},
		Sect_Exclusions: {
			IncludedPath: 'list',
			ExcludedPath: 'list',
		},
	},
	SMBSpider: {
		Sect_General: {
			Start: 'bool',
			AlertFiles: 'bool',
			ActionDelay: 'number',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Scansettings: {
			OnKnownVirus: ['Cure', 'Quarantine', 'Delete', 'Block'],
			OnIncurable: ['Quarantine', 'Delete', 'Block'],
			OnSuspicious: ['Pass', 'Block', 'Quarantine', 'Delete'],
			OnAdware: ['Pass', 'Block', 'Quarantine', 'Delete'],
			OnDialers: ['Pass', 'Block', 'Quarantine', 'Delete'],
			OnJokes: ['Pass', 'Block', 'Quarantine', 'Delete'],
			OnRiskware: ['Pass', 'Block', 'Quarantine', 'Delete'],
			OnHacktools: ['Pass', 'Block', 'Quarantine', 'Delete'],
			BlockOnError: 'bool',
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			ArchiveMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')
			},
			MailMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')
			},
			PackerMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')
			},
			ContainerMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')
			},
			MaxCompressionRatio: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')
			},
		},
		Sect_Exclusions: {
			IncludedPath: 'list',
			ExcludedPath: 'list',
		},
		Sect_Advanced: {
			SmbSocketPath: 'string',
			SambaChrootDir: 'string',
			MaxCacheSize: 'number',
		},
	},
	Update: {
		Sect_General: {
			RunAsUser: 'string',
			Proxy: 'string',
			BaseUpdateEnabled: 'bool',
			DwsUpdateEnabled: 'bool',
			AntispamUpdateEnabled: 'bool',
			VersionUpdateEnabled: 'bool',
			UpdateInterval: 'number',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Advanced: {
			NetworkTimeout: 'number',
			RetryInterval: 'number',
			MaxRetries: 'number',
			BackupDir: 'string',							
			NumberOfBackups: 'number',
		},
		Sect_Exclusions: {
			ExcludedFiles: 'list',
		},
		Sect_Zones: {
			BaseDrlPath: 'string',
			BaseCustomDrlPath: 'string',
			DwsDrlPath: 'string',
			DwsCustomDrlPath: 'string',
			AntispamDrlPath: 'string',
			AntispamCustomDrlPath: 'string',
			VersionDrlPath: 'string',
		},
	},
	LinuxGUI: {
		Sect_Scansettings: {
			OnKnownVirus: ['Quarantine', 'Delete'],
			OnSuspicious: ['Report', 'Quarantine', 'Delete'],
			OnAdware: ['Report', 'Quarantine', 'Delete'],
			OnDialers: ['Report', 'Quarantine', 'Delete'],
			OnJokes: ['Report', 'Quarantine', 'Delete'],
			OnRiskware: ['Report', 'Quarantine', 'Delete'],
			OnHacktools: ['Report', 'Quarantine', 'Delete'],
			CheckArchives: 'bool',
			CheckEmails: 'bool',
			AutoApplying: 'bool',
			ScanTimeout: 'number',
		},
		Sect_Exclusions: {
			ExcludedPath: 'list',
		}
	},
	// LinuxFirewall.InputDivert "Auto(interface:enp0s7 protected: 80)"
	// LinuxFirewall.InputDivert "Proxy(interface:enp0s7 protected: 80)"
	LinuxFirewall: {
		Sect_General: {
			OutputDivert: 'divert',
			InputDivert: 'divert',
			UnwrapSsl: 'bool',
			BlockUnchecked: 'bool',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
			InspectHttp: 'switcher',
			InspectPop3: 'switcher',
			InspectImap: 'switcher',
			InspectSmtp: 'switcher',
		},
		Sect_Webfilter: {
			HttpCheckAddress: 'bool',
			HttpCheckContent: 'bool',
			BlockNotRecommended: 'bool',
			BlockAdultContent: 'bool',
			BlockViolence: 'bool',
			BlockWeapons: 'bool',
			BlockGambling: 'bool',
			BlockDrugs: 'bool',
			BlockObsceneLanguage: 'bool',
			BlockChats: 'bool',
			BlockTerrorism: 'bool',
			BlockFreeEmail: 'bool',
			BlockSocialNetworks: 'bool',
			BlockDueToCopyrightNotice: 'bool',
		},
		Sect_Exclusions: {
			Whitelist: 'list',
			Blacklist: 'list',
			ExcludedProc: 'list',
		},
		Sect_Mailfilter: {
			SmtpCheckContent: 'bool',
			Pop3CheckContent: 'bool',
			ImapCheckContent: 'bool',
		},
		Sect_Filefilter: {
			BlockKnownVirus: 'bool',
			BlockSuspicious: 'bool',
			BlockAdware: 'bool',
			BlockDialers: 'bool',
			BlockJokes: 'bool',
			BlockRiskware: 'bool',
			BlockHacktools: 'bool',
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			ArchiveMaxLevel: 'number',
			MailMaxLevel: 'number',
			PackerMaxLevel: 'number',
			ContainerMaxLevel: 'number',
			MaxCompressionRatio: 'number',
		},
		Sect_Ruleset: {
			RuleSet0: 'list',
			RuleSet1: 'list',
			RuleSet2: 'list',
			RuleSet3: 'list',
			RuleSet4: 'list',
			RuleSet5: 'list',
			RuleSet6: 'list',
			RuleSet7: 'list',
			RuleSet8: 'list',
			RuleSet9: 'list',
			RuleSet10: 'list',
		}
	},
	ICAPD: {
		Sect_General: {
			Start: 'bool',
			RunAsUser: 'string',
			ListenAddress: 'string',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Advanced: {
			UsePreview: 'bool',
			Use204: 'bool',
			AllowEarlyResponse: 'bool',
			HtmlTemplatesDir: 'string',
		},
		Sect_Webfilter: {
			BlockNotRecommended: 'bool',
			BlockAdultContent: 'bool',
			BlockViolence: 'bool',
			BlockWeapons: 'bool',
			BlockGambling: 'bool',
			BlockDrugs: 'bool',
			BlockObsceneLanguage: 'bool',
			BlockChats: 'bool',
			BlockTerrorism: 'bool',
			BlockFreeEmail: 'bool',
			BlockSocialNetworks: 'bool',
			BlockDueToCopyrightNotice: 'bool',
		},
		Sect_Exclusions: {
			Whitelist: 'list',
			Blacklist: 'list',
		},
		Sect_Filefilter: {
			BlockKnownVirus: 'bool',
			BlockSuspicious: 'bool',
			BlockAdware: 'bool',
			BlockDialers: 'bool',
			BlockJokes: 'bool',
			BlockRiskware: 'bool',
			BlockHacktools: 'bool',
			BlockUnchecked: 'bool',
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			ArchiveMaxLevel: 'number',
			MailMaxLevel: 'number',
			PackerMaxLevel: 'number',
			ContainerMaxLevel: 'number',
			MaxCompressionRatio: 'number',
		},
		Sect_Ruleset: {
			RuleSet0: 'list',
			RuleSet1: 'list',
			RuleSet2: 'list',
			RuleSet3: 'list',
			RuleSet4: 'list',
			RuleSet5: 'list',
			RuleSet6: 'list',
		},
		Sect_Debug:{
			DebugDumpIcap: 'bool',
		},
	},
	HTTPD: {
		Sect_General: {
			WebConsoleAddress: 'list', // Depricated since 11.1
			AdminListen: 'list', // since 11.1
			LinkCheckerAddress: 'list', // Depricated since 11.1
			PublicListen: 'list', // since 11.1
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_SSL: {
			ServerSslCertificate: 'string', // Depricated since 11.1
			AdminSslCertificate: 'string', // since 11.1
			ServerSslKey: 'string', // Depricated since 11.1
			AdminSslKey: 'string', // since 11.1
			AdminSslCa: 'string', // since 11.1
		},
		Sect_Advanced: {
			AccessLogPath: 'string',
			HelpRoot: 'string',
			WebConsoleRoot: 'string',
			LinkCheckerRoot: 'string',
		},
	},
	GateD: {
		General:{
			RunAsUser: 'string',
			IdleTimeLimit: 'number',	
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Paths: {
			HtmlTemplatesDir: 'string',
			CaDir: 'string',
		},
		Sect_Debug: {
			DebugSocketPath: 'string',	
		},
	},
	FileCheck: {
		Sect_General: {
			RescanInterval: 'number',
			IdleTimeLimit: 'number',
			MaxCacheSize: 'number',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Debug: {
			DebugClientIpc: 'bool',			
			DebugScan: 'bool',
			DebugFlowScan: 'bool',
			DebugProxyScan: 'bool',
			DebugCache: 'bool',
		},
	},
	ESAgent: {
		Sect_General: {
			MobileMode: ['On', 'Off', 'Auto'],
			Discovery: 'bool',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Debug: {
			DebugIpc: 'bool',
		},
	},
	Root: {
		Sect_General: {
			AdminGroup: 'string',
			TrustedGroup: 'string',
			UseCloud: 'bool',
			DefaultLogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: {
				type: 'string',
				pattern: '^\/.*$|^Syslog\\:.+$|^Stderr.+$',
				warning: 'pattern_Root.Log',
			},
		},
		Sect_Paths: {
			PublicSocketPath: 'string',
			AdminSocketPath: 'string',
			RunDir: 'string',
			TempDir: 'string',
			CacheDir: 'string',
			CoreEnginePath: 'string',
			KeyPath: 'string',
			VirusBaseDir: 'string',
			VarLibDir: 'string',
			DwsDir: 'string',
			VersionDir: 'string',
			AntispamCorePath: 'string',
			AntispamEngineExePath: 'string',
		},
		Sect_Debug: {
			DebugIpc: 'bool',
		}
	},
	CloudD: {
		Sect_General: {
			RunAsUser: 'string',
			IdleTimeLimit: 'number',
			PersistentCache: 'bool',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Debug: {
			DebugSdk: 'bool',
		}
	},
	ClamD: {
		Sect_General: {
			Start: 'bool',
			LogLevel: ['Debug', 'Info', 'Notice', 'Warning', 'Error'],
			Log: 'log',
		},
		Sect_Threats: {
			DetectSuspicious: 'bool',
			DetectAdware: 'bool',
			DetectDialers: 'bool',
			DetectJokes: 'bool',
			DetectRiskware: 'bool',
			DetectHacktools: 'bool',
		},
		Sect_Advanced: {
			ReadTimeout: 'number',
			StreamMaxLength: 'number',
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			ArchiveMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			MailMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			PackerMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			ContainerMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			MaxCompressionRatio: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
		}
	},
	ClamD_Endpoint:{
		Sect_General: {
			ClamdSocket: 'string',
			ReadTimeout: 'number',
			StreamMaxLength: 'number',
		},
		Sect_Threats: {
			DetectSuspicious: 'bool',
			DetectAdware: 'bool',
			DetectDialers: 'bool',
			DetectJokes: 'bool',
			DetectRiskware: 'bool',
			DetectHacktools: 'bool',
		},
		Sect_Advanced: {
			ScanTimeout: 'number',
			HeuristicAnalysis: 'switcher',
			ArchiveMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			MailMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			PackerMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			ContainerMaxLevel: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
			MaxCompressionRatio: {
				type: 'number',
				emptyMes: App.helpers._('Do_not_check')	
			},
		},
	},
	Console: {
		Sect_General: {
			Lang: '#items_Lang',
			MaxCheckFileSize: {
				type: 'number',
				postfix: App.helpers._('vol_Mb')
			}
		}
	},
};
SCHEME.MacSpider =  SCHEME.LinuxSpider;
SCHEME.MacFirewall =  SCHEME.LinuxFirewall;
SCHEME.MacGUI = SCHEME.LinuxGUI;
SCHEME.SMBSpider_Share = SCHEME.SMBSpider;

	return App.view.settings;
});	